/* ************************************************************************
 *   File: act.comm.c                                    Part of CircleMUD *
 *  Usage: Player-level communication commands                             *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "mobprog.h"
#include "screen.h"


/* extern variables */
extern int level_can_shout;
extern int holler_move_cost;
extern int top_of_p_table;
extern struct player_index_element *player_table;

/* local functions */
void perform_tell(struct char_data *ch, struct char_data *vict, char *arg);
int  is_tell_ok(struct char_data *ch, struct char_data *vict);
char *makedrunk(char *string ,struct char_data *ch);

ACMD(do_contact);
ACMD(do_gen_comm);
ACMD(do_gsay);
ACMD(do_page);
ACMD(do_priv_chan);
ACMD(do_qcomm);
ACMD(do_reply);
ACMD(do_say);
ACMD(do_spec_comm);
ACMD(do_tell);
ACMD(do_write);


void search_replace(char *string, const char *find, const char *replace)
{
	char final[MAX_INPUT_LENGTH], temp[2];
	size_t start, end, i;

	while (strstr(string, find) != NULL) {
		final[0] = '\0';
		start = strstr(string, find) - string;
		end = start + strlen(find);

		temp[1] = '\0';
		strncat(final, string, start);
		strcat(final, replace);

		for (i = end; string[i] != '\0'; i++) {
			temp[0] = string[i];
			strcat(final, temp);
		}

		sprintf(string, final);
	}
}


ACMD(do_say)
{
	int face, state, len, len2;
	char buf[MAX_STRING_LENGTH] = "", buf2[MAX_STRING_LENGTH] = "";

	skip_spaces(&argument);
	
	if (!*argument) {
		send_to_char(ch, "Yes, but WHAT do you want to say?\r\n");
		return;
	}
	
	argument = makedrunk(argument, ch);

	snprintf(buf, sizeof(buf), "$n ");   len = strlen(buf);
	snprintf(buf2, sizeof(buf), "You "); len2 = strlen(buf2);
	
	if (strstr(argument, ":)") || strstr(argument, ":-)") ||
		 strstr(argument, ":>") || strstr(argument, ":->"))
		face = 0;
	else if (strstr(argument, ";)") || strstr(argument, ";-)") ||
				strstr(argument, ";>") || strstr(argument, ";->"))
		face = 1;
	else if (strstr(argument, ";(") || strstr(argument, ";-(") ||
				strstr(argument, ";<") || strstr(argument, ";-<"))
		face = 2;
	else if (strstr(argument, ":(") || strstr(argument, ":-(") ||
				strstr(argument, ";<") || strstr(argument, ";-<"))
		face = 2;
	
	search_replace(argument, " :)", ""); search_replace(argument, " :-)", "");
	search_replace(argument, " :>", ""); search_replace(argument, " :->", "");
	search_replace(argument, " ;)", ""); search_replace(argument, " ;-)", "");
	search_replace(argument, " ;>", ""); search_replace(argument, " ;->", "");
	search_replace(argument, " ;(", ""); search_replace(argument, " ;-(", "");
	search_replace(argument, " ;<", ""); search_replace(argument, " ;-<", "");
	search_replace(argument, " :(", ""); search_replace(argument, " :-(", "");
	search_replace(argument, " :<", ""); search_replace(argument, " :-<", "");
	search_replace(argument, ":)", "");  search_replace(argument, ":-)", "");
	search_replace(argument, ":>", "");  search_replace(argument, ":->", "");
	search_replace(argument, ";)", "");  search_replace(argument, ";-)", "");
	search_replace(argument, ";>", "");  search_replace(argument, ";->", "");
	search_replace(argument, ";(", "");  search_replace(argument, ";-(", "");
	search_replace(argument, ";<", "");  search_replace(argument, ";-<", "");
	search_replace(argument, ":(", "");  search_replace(argument, ":-(", "");
	search_replace(argument, ":<", "");  search_replace(argument, ":-<", "");
  
	if (!*argument) {
		send_to_char(ch, "Yes, but WHAT do you want to say?\r\n");
		return;
	}

	if (argument[strlen(argument)-1] != '.' && 
		 argument[strlen(argument)-1] != '?' &&
		 argument[strlen(argument)-1] != '!')
		strcat(argument, ".");

	if (strstr(argument, "?"))
		state = 1;
	else if (strstr(argument, "!"))
		state = 2;
	else
		state = 0;

	switch(face) {
		case 0:
			snprintf(buf + len, sizeof(buf) - len, "smiles and ");
			snprintf(buf2 + len2, sizeof(buf2) - len2, "smile and ");
			break;
		case 1:
			snprintf(buf + len, sizeof(buf) - len, "winks and ");
			snprintf(buf2 + len2, sizeof(buf2) - len2, "wink and ");
			break;
		case 2:
			snprintf(buf + len, sizeof(buf) - len, "frowns and ");
			snprintf(buf2 + len2, sizeof(buf2) - len2, "frown and ");
			break;
		default:
			break;
	}
	len  = strlen(buf);
	len2 = strlen(buf2);
        
	switch(state) {
		case 1:
			snprintf(buf + len, sizeof(buf) - len, "asks, \"%s\"", argument);
			snprintf(buf2 + len2, sizeof(buf2) - len2, "ask, \"%s\"", argument);
			break;
		case 2:
			snprintf(buf + len, sizeof(buf) - len, "exclaims, \"%s\"", argument);
			snprintf(buf2 + len2, sizeof(buf2) - len2, "exclaim, \"%s\"", argument);
			break;
		default:
		case 0:
			snprintf(buf + len, sizeof(buf) - len, "says, \"%s\"", argument);
			snprintf(buf2 + len2, sizeof(buf2) - len2, "say, \"%s\"", argument);
			break;
	}
	
	MOBTrigger = FALSE;
	act(buf, FALSE, ch, 0, 0, TO_ROOM);
	
	if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_NOREPEAT))
		send_to_char(ch, "%s", OK);
	else
		act(buf2, FALSE, ch, 0, argument, TO_CHAR);
	
	mprog_speech_trigger(argument, ch);
}


#if 0
/* If you don't want the emoticon thingy for do_say, just delete the
 * above version of the function and uncomment out this one.  -ejg */
ACMD(do_say)
{
	skip_spaces(&argument);

	if (!*argument)
		send_to_char(ch, "Yes, but WHAT do you want to say?\r\n");
	else {
		char buf[MAX_INPUT_LENGTH + 12];
		argument = makedrunk(argument, ch);

		snprintf(buf, sizeof(buf), "$n says, '%s'", argument);
		MOBTrigger = FALSE;
		act(buf, FALSE, ch, 0, 0, TO_ROOM);

		if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else {
			delete_doubledollar(argument);
			send_to_char(ch, "You say, '%s'\r\n", argument);
		}
		mprog_speech_trigger(argument, ch);
	}
}
#endif


ACMD(do_gsay)
{
	struct char_data *k;
	struct follow_type *f;

	skip_spaces(&argument);

	if (!AFF_FLAGGED(ch, AFF_GROUP)) {
		send_to_char(ch, "But you are not the member of a group!\r\n");
		return;
	}
	if (!*argument)
		send_to_char(ch, "Yes, but WHAT do you want to group-say?\r\n");
	else {
		char buf[MAX_STRING_LENGTH];

		if (ch->master)
			k = ch->master;
		else
			k = ch;

		argument = makedrunk(argument, ch);
		snprintf(buf, sizeof(buf), "$n tells the group, '%s'", argument);

		if (AFF_FLAGGED(k, AFF_GROUP) && (k != ch))
			act(buf, FALSE, ch, 0, k, TO_VICT | TO_SLEEP);
		for (f = k->followers; f; f = f->next)
			if (AFF_FLAGGED(f->follower, AFF_GROUP) && (f->follower != ch))
				act(buf, FALSE, ch, 0, f->follower, TO_VICT | TO_SLEEP);

		if (PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else
			send_to_char(ch, "You tell the group, '%s'\r\n", argument);
	}
}


void perform_tell(struct char_data *ch, struct char_data *vict, char *arg)
{
	char buf[MAX_STRING_LENGTH];

	send_to_char(vict, "%s", CCRED(vict, C_NRM));
	arg = makedrunk(arg, ch);
	snprintf(buf, sizeof(buf), "$n tells you, '%s'", arg);
	act(buf, FALSE, ch, 0, vict, TO_VICT | TO_SLEEP);
	send_to_char(vict, "%s", CCNRM(vict, C_NRM));

	if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_NOREPEAT))
		send_to_char(ch, "%s", OK);
	else {
		send_to_char(ch, "%s", CCRED(ch, C_CMP));
		snprintf(buf, sizeof(buf), "You tell $N, '%s'", arg);
		act(buf, FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
		send_to_char(ch, "%s", CCNRM(ch, C_CMP));
	}

	if (!IS_NPC(vict) && !IS_NPC(ch))
		GET_LAST_TELL(vict) = GET_IDNUM(ch);
}

int is_tell_ok(struct char_data *ch, struct char_data *vict)
{
	if (ch == vict)
		send_to_char(ch, "You try to tell yourself something.\r\n");
	else if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_NOTELL))
		send_to_char(ch, "You can't tell other people while you have "
						 "notell on.\r\n");
	else if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_SOUNDPROOF))
		send_to_char(ch, "The walls seem to absorb your words.\r\n");
	else if (!IS_NPC(vict) && !vict->desc)        /* linkless */
		act("$E's linkless at the moment.", 
			 FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
	else if (!IS_NPC(vict) && EDITING(vict->desc))
		act("$E's writing a message right now; try again later.", 
			 FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
	else if ((!IS_NPC(vict) && PRF_FLAGGED(vict, PRF_NOTELL)) || 
				ROOM_FLAGGED(IN_ROOM(vict), ROOM_SOUNDPROOF))
		act("$E can't hear you.", FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
	else if (PRF_FLAGGED(vict, PRF_AFK))
		act("$E's afk right now, try again later.",
			 FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
	else
		return (TRUE);

	return (FALSE);
}

/*
 * Yes, do_tell probably could be combined with whisper and ask, but
 * called frequently, and should IMHO be kept as tight as possible.
 */
ACMD(do_tell)
{
	struct char_data *vict = NULL;
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];

	half_chop(argument, buf, buf2);

	if (!*buf || !*buf2)
		send_to_char(ch, "Who do you wish to tell what??\r\n");

	else if (GET_LEVEL(ch) < LVL_IMMORT && 
				!(vict = get_player_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
		send_to_char(ch, "%s", NOPERSON);

	else if (GET_LEVEL(ch) >= LVL_IMMORT && 
				!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
		send_to_char(ch, "%s", NOPERSON);

	else if (is_tell_ok(ch, vict))
		perform_tell(ch, vict, buf2);
}


ACMD(do_reply)
{
	struct char_data *tch = character_list;

	if (IS_NPC(ch))
		return;

	skip_spaces(&argument);

	if (GET_LAST_TELL(ch) == NOBODY)
		send_to_char(ch, "You have nobody to reply to!\r\n");
	else if (!*argument)
		send_to_char(ch, "What is your reply?\r\n");
	else {
		/*
		 * Make sure the person you're replying to is still playing by searching
		 * for them.  Note, now last tell is stored as player IDnum instead of
		 * a pointer, which is much better because it's safer, plus will still
		 * work if someone logs out and back in again.
		 */
				     
		/*
		 * XXX: A descriptor list based search would be faster although
		 *      we could not find link dead people.  Not that they can
		 *      hear tells anyway. :) -gg 2/24/98
		 */
		while (tch != NULL && (IS_NPC(tch) || GET_IDNUM(tch) != GET_LAST_TELL(ch)))
			tch = tch->next;

		if (tch == NULL)
			send_to_char(ch, "They are no longer playing.\r\n");
		else if (is_tell_ok(ch, tch))
		{
			argument = makedrunk(argument, ch);
			perform_tell(ch, tch, argument);
		}
	}
}


ACMD(do_spec_comm)
{
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];
	struct char_data *vict;
	const char *action_sing, *action_plur, *action_others;

	switch (subcmd) {
		case SCMD_WHISPER:
			action_sing = "whisper to";
			action_plur = "whispers to";
			action_others = "$n whispers something to $N.";
			break;

		case SCMD_ASK:
			action_sing = "ask";
			action_plur = "asks";
			action_others = "$n asks $N a question.";
			break;

		default:
			action_sing = "oops";
			action_plur = "oopses";
			action_others = "$n is tongue-tied trying to speak with $N.";
			break;
	}

	half_chop(argument, buf, buf2);

	if (!*buf || !*buf2)
		send_to_char(ch, "Whom do you want to %s.. and what??\r\n", action_sing);
	else if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM)))
		send_to_char(ch, "%s", NOPERSON);
	else if (vict == ch)
		send_to_char(ch, "You can't get your mouth close enough to your ear...\r\n");
	else {
		char buf1[MAX_STRING_LENGTH];

		argument = makedrunk(buf2, ch);
		snprintf(buf1, sizeof(buf1), "$n %s you, '%s'", action_plur, argument);
		act(buf1, FALSE, ch, 0, vict, TO_VICT);

		if (PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else
			send_to_char(ch, "You %s %s, '%s'\r\n", action_sing, GET_NAME(vict), buf2);
		act(action_others, FALSE, ch, 0, vict, TO_NOTVICT);
	}
}


/*
 * buf1, buf2 = MAX_OBJECT_NAME_LENGTH
 *	(if it existed)
 */
ACMD(do_write)
{
	struct obj_data *paper, *pen = NULL;
	char *papername, *penname;
	char buf1[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];

	papername = buf1;
	penname = buf2;

	two_arguments(argument, papername, penname);

	if (!ch->desc)
		return;

	if (!*papername) {		/* nothing was delivered */
		send_to_char(ch, "Write?  With what?  ON what?  What are you trying to do?!?\r\n");
		return;
	}
	if (*penname) {		/* there were two arguments */
		if (!(paper = get_obj_in_list_vis(ch, papername, NULL, ch->carrying))) {
			send_to_char(ch, "You have no %s.\r\n", papername);
			return;
		}
		if (!(pen = get_obj_in_list_vis(ch, penname, NULL, ch->carrying))) {
			send_to_char(ch, "You have no %s.\r\n", penname);
			return;
		}
	} else {		/* there was one arg.. let's see what we can find */
		if (!(paper = get_obj_in_list_vis(ch, papername, NULL, ch->carrying))) {
			send_to_char(ch, "There is no %s in your inventory.\r\n", papername);
			return;
		}
		if (GET_OBJ_TYPE(paper) == ITEM_PEN) {	/* oops, a pen.. */
			pen = paper;
			paper = NULL;
		} else if (GET_OBJ_TYPE(paper) != ITEM_NOTE) {
			send_to_char(ch, "That thing has nothing to do with writing.\r\n");
			return;
		}
		/* One object was found.. now for the other one. */
		if (!GET_EQ(ch, WEAR_HOLD)) {
			send_to_char(ch, "You can't write with %s %s alone.\r\n", 
							 AN(papername), papername);
			return;
		}
		if (!CAN_SEE_OBJ(ch, GET_EQ(ch, WEAR_HOLD))) {
			send_to_char(ch, "The stuff in your hand is invisible!  Yeech!!\r\n");
			return;
		}
		if (pen)
			paper = GET_EQ(ch, WEAR_HOLD);
		else
			pen = GET_EQ(ch, WEAR_HOLD);
	}


	/* ok.. now let's see what kind of stuff we've found */
	if (GET_OBJ_TYPE(pen) != ITEM_PEN)
		act("$p is no good for writing with.", FALSE, ch, pen, 0, TO_CHAR);
	else if (GET_OBJ_TYPE(paper) != ITEM_NOTE)
		act("You can't write on $p.", FALSE, ch, paper, 0, TO_CHAR);

	/* Delete this part to allow people to edit notes */
	else if (paper->action_description)
		send_to_char(ch, "There's something written on it already.\r\n");
	else {
		/* we can write - hooray! */
		send_to_char(ch, "Write your note.  End with '@' on a new line.\r\n");
		act("$n begins to jot down a note.", TRUE, ch, 0, 0, TO_ROOM);
		string_write(ch->desc, &paper->action_description, 
						 MAX_NOTE_LENGTH, 0, NOTHING, EDIT_NOTE);
		GET_OBJ_EXTRA(paper).ITEM_UNIQUE_SAVE = 1;
	}
}



ACMD(do_page)
{
	struct descriptor_data *d;
	struct char_data *vict;
	char buf2[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];

	half_chop(argument, arg, buf2);

	if (IS_NPC(ch))
		send_to_char(ch, "Monsters can't page.. go away.\r\n");
	else if (!*arg)
		send_to_char(ch, "Whom do you wish to page?\r\n");
	else {
		char buf[MAX_STRING_LENGTH];

		snprintf(buf, sizeof(buf), "\007\007*$n* %s", buf2);
		if (!str_cmp(arg, "all")) {
			if (GET_LEVEL(ch) > LVL_GOD) {
				for (d = descriptor_list; d; d = d->next)
					if (STATE(d) == CON_PLAYING && d->character)
						act(buf, FALSE, ch, 0, d->character, TO_VICT);
			} else
				send_to_char(ch, "You will never be godly enough to do that!\r\n");
			return;
		}
		if ((vict = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD)) != NULL) {
			act(buf, FALSE, ch, 0, vict, TO_VICT);
			if (PRF_FLAGGED(ch, PRF_NOREPEAT))
				send_to_char(ch, "%s", OK);
			else
				act(buf, FALSE, ch, 0, vict, TO_CHAR);
		} else
			send_to_char(ch, "There is no such person in the game!\r\n");
	}
}


/**********************************************************************
 * generalized communication func, originally by Fred C. Merkel (Torg) *
 *********************************************************************/
int on_channel(struct char_data *ch, int channel)
{
	switch(channel) {
		case SCMD_HOLLER:
			return 1;
		case SCMD_SHOUT:
			return !PRF_FLAGGED(ch, PRF_DEAF);
		case SCMD_GOSSIP:
			return !PRF_FLAGGED(ch, PRF_NOGOSS);
		case SCMD_AUCTION:
			return !PRF_FLAGGED(ch, PRF_NOAUCT);
		case SCMD_GRATZ:
			return !PRF_FLAGGED(ch, PRF_NOGRATZ);
		default:
			log("NON EXISTING CHANNEL IN on_channel comm.c");
			return 0;
	}
}


ACMD(do_gen_comm)
{
	struct descriptor_data *i;
	char color_on[24];
	char buf1[MAX_INPUT_LENGTH];

	/*
	 * com_msgs: [0] Message if you can't perform the action because of noshout
	 *           [1] name of the action
	 *           [2] message if you're not on the channel
	 *           [3] a color string.
	 */
	const char *com_msgs[][4] = {
		{"You cannot holler!!\r\n",
		 "holler",
		 "",
		 KYEL},

		{"You cannot shout!!\r\n",
		 "shout",
		 "Turn off your noshout flag first!\r\n",
		 KYEL},

		{"You cannot gossip!!\r\n",
		 "gossip",
		 "You aren't even on the channel!\r\n",
		 KYEL},

		{"You cannot auction!!\r\n",
		 "auction",
		 "You aren't even on the channel!\r\n",
		 KMAG},

		{"You cannot congratulate!\r\n",
		 "congrat",
		 "You aren't even on the channel!\r\n",
		 KGRN},

		{"You cannot talk on the Private Channel!\r\n",
		 "psay",
		 "You aren't even on the channel!\r\n",
		 KYEL},
	};

	if (subcmd == SCMD_PRIVATE && GET_PRIVATE(ch) == 0) {
		send_to_char(ch, "You are not on a private channel!");
		return;
	}

	/* to keep pets, etc from being ordered to shout */
	if (!ch->desc)
		return;

	if (PLR_FLAGGED(ch, PLR_NOSHOUT)) {
		send_to_char(ch, "%s", com_msgs[subcmd][0]);
		return;
	}
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_SOUNDPROOF)) {
		send_to_char(ch, "The walls seem to absorb your words.\r\n");
		return;
	}
	/* level_can_shout defined in config.c */
	if (GET_LEVEL(ch) < level_can_shout) {
		send_to_char(ch, "You must be at least level %d before you can %s.\r\n", 
						 level_can_shout, com_msgs[subcmd][1]);
		return;
	}
	/* make sure the char is on the channel */
	if (!on_channel(ch, subcmd)) {
		send_to_char(ch, "%s", com_msgs[subcmd][2]);
		return;
	}
	/* skip leading spaces */
	skip_spaces(&argument);

	/* make sure that there is something there to say! */
	if (!*argument) {
		send_to_char(ch, "Yes, %s, fine, %s we must, but WHAT???\r\n", 
						 com_msgs[subcmd][1], com_msgs[subcmd][1]);
		return;
	}
	if (subcmd == SCMD_HOLLER) {
		if (GET_MOVE(ch) < holler_move_cost) {
			send_to_char(ch, "You're too exhausted to holler.\r\n");
			return;
		} else
			GET_MOVE(ch) -= holler_move_cost;
	}

	argument = makedrunk(argument,ch);
	/* set up the color on code */
	strlcpy(color_on, com_msgs[subcmd][3], sizeof(color_on));

	/* first, set up strings to be given to the communicator */
	if (PRF_FLAGGED(ch, PRF_NOREPEAT))
		send_to_char(ch, "%s", OK);
	else
		send_to_char(ch, "%sYou %s, '%s'%s", 
						 COLOR_LEV(ch) >= C_CMP ? color_on : "",
						 com_msgs[subcmd][1], argument, CCNRM(ch, C_CMP));

	snprintf(buf1, sizeof(buf1), "$n %ss, '%s'", com_msgs[subcmd][1], argument);

	/* now send all the strings out */
	for (i = descriptor_list; i; i = i->next) {
		if (STATE(i) == CON_PLAYING && i != ch->desc && i->character &&
			 on_channel(i->character, subcmd) &&
			 !EDITING(i) &&
			 !ROOM_FLAGGED(IN_ROOM(i->character), ROOM_SOUNDPROOF)) {

			if (subcmd == SCMD_PRIVATE &&
				 (GET_PRIVATE(ch) != GET_PRIVATE(i->character)))
				continue;

			if (subcmd == SCMD_SHOUT &&
				 ((world[IN_ROOM(ch)].zone != world[IN_ROOM(i->character)].zone) ||
				  !AWAKE(i->character)))
				continue;

			if (COLOR_LEV(i->character) >= C_NRM)
				send_to_char(i->character, "%s", color_on);
			act(buf1, FALSE, ch, 0, i->character, TO_VICT | TO_SLEEP);
			if (COLOR_LEV(i->character) >= C_NRM)
				send_to_char(i->character, "%s", KNRM);
		}
	}
}


ACMD(do_qcomm)
{
	if (!PRF_FLAGGED(ch, PRF_QUEST)) {
		send_to_char(ch, "You aren't even part of the quest!\r\n");
		return;
	}
	skip_spaces(&argument);

	if (!*argument)
		send_to_char(ch, "%c%s?  Yes, fine, %s we must, but WHAT??\r\n", 
						 UPPER(*CMD_NAME), CMD_NAME + 1, CMD_NAME);
	else {
		char buf[MAX_STRING_LENGTH];
		struct descriptor_data *i;

		argument = makedrunk(argument, ch);

		if (PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else if (subcmd == SCMD_QSAY) {
			snprintf(buf, sizeof(buf), "You quest-say, '%s'", argument);
			act(buf, FALSE, ch, 0, argument, TO_CHAR);
		} else
			act(argument, FALSE, ch, 0, argument, TO_CHAR);

		if (subcmd == SCMD_QSAY)
			snprintf(buf, sizeof(buf), "$n quest-says, '%s'", argument);
		else
			strlcpy(buf, argument, sizeof(buf));

		for (i = descriptor_list; i; i = i->next)
			if (STATE(i) == CON_PLAYING && 
				 i != ch->desc &&
				 PRF_FLAGGED(i->character, PRF_QUEST))
				act(buf, 0, ch, 0, i->character, TO_VICT | TO_SLEEP);
	}
}


struct char_data *find_char(long n)
{
	int i;
	struct char_data *cur = character_list;
	
	for (i = 0; i <= top_of_p_table; i++)
		if (player_table[i].id == n)
			break;
		
	while (cur)
	{
		if (GET_PFILEPOS(cur) == i)
			return cur;
		
		cur = cur->next;
	}
	
	return NULL;
}


ACMD(do_contact)
{
	struct char_data *contact;
	char buf[MAX_STRING_LENGTH];
  
	if (!IS_NPC(ch) && GET_CONTACT(ch)) {
		if ((contact = find_char(get_id_by_name(GET_CONTACT(ch)))) != NULL) {
			if (STATE(contact->desc) == CON_PLAYING) {
				snprintf(buf, sizeof(buf), "%s is trying to contact you.\r\n",
							GET_NAME(ch));
				send_to_char(contact, buf);

				snprintf(buf, sizeof(buf), "You attempt to contact %s.\r\n", 
							GET_NAME(contact));
				send_to_char(ch, buf);
			}
		}
	} else
		send_to_char(ch, "Your calls seem to go unanswered.\r\n");
}


ACMD(do_priv_chan)
{
	struct char_data *vict;
	struct descriptor_data *i;
	char buf[MAX_INPUT_LENGTH], buf2[MAX_STRING_LENGTH];

	half_chop(argument, buf, buf2);

	if (subcmd == PRIVATE_HELP) {
		send_to_char(ch, "Private Channel (PC) commands\r\n");
		send_to_char(ch, "-----------------------------\r\n");
		send_to_char(ch, "popen   - opens your own private channel.\r\n");
		send_to_char(ch, "padd    - adds a player to your PC.\r\n");
		send_to_char(ch, "premove - remove a player from your PC.\r\n");
		send_to_char(ch, "pclose  - closes your private channel.\r\n");
		send_to_char(ch, "pwho    - lists all members on the current PC.\r\n");
		send_to_char(ch, "poff    - exits you from your current PC.\r\n\r\n");
		send_to_char(ch, "phelp   - this screen.\r\n\r\n");
		send_to_char(ch, "  NOTE: If you don't want to be added to another\r\n");
		send_to_char(ch, "        player's PC open your own with no players.\r\n");
		send_to_char(ch, "\r\nTo talk on the channel use -- private, psay or .\r\n");
	} else if (subcmd == PRIVATE_OPEN) {
		GET_PRIVATE(ch) = GET_IDNUM(ch);
		send_to_char(ch, "You have just opened your own Private Channel.\r\n");
	} else if (subcmd == PRIVATE_OFF) {
		GET_PRIVATE(ch) = 0;
		send_to_char(ch, "You have just quit any Private Channels.\r\n");
	} else if (subcmd == PRIVATE_CLOSE) {
		GET_PRIVATE(ch) = 0;
		/* now remove all people on the private channel */
		for (i = descriptor_list; i; i = i->next)
			if (!i->connected)
				if ((GET_PRIVATE(i->character) == GET_IDNUM(ch)) &&
					 (ch != i->character)) {
					GET_PRIVATE(i->character) = 0;
					send_to_char(i->character, "%s has just closed their Private "
									 "Channel.\r\n", GET_NAME(ch));
				}
		send_to_char(ch, "You have just CLOSED your Private Channel.\r\n");
	} else if (subcmd == PRIVATE_WHO) {
		if (GET_PRIVATE(ch) == 0)
			send_to_char(ch, "You are not on a private channel\r\n");
		else {
			/* show all people on the private channel */
			send_to_char(ch, "Private Channel Members\r\n");
			send_to_char(ch, "-----------------------\r\n");
			for (i = descriptor_list; i; i = i->next)
				if (!i->connected)
					if (GET_PRIVATE(i->character) == GET_PRIVATE(ch))
						send_to_char(ch, "%s\r\n", GET_NAME(i->character));
		}
	} else if (subcmd == PRIVATE_CHECK) {
      /* show all people on the ALL private channels */
      send_to_char(ch, "Private Channels\r\n");
      send_to_char(ch, "---------------------------------------------\r\n");
      for (i = descriptor_list; i; i = i->next)
			if (!i->connected) 
				send_to_char(ch, "[%-5d]  %s\r\n", 
								 GET_PRIVATE(i->character), 
								 GET_NAME(i->character));
	} else if (subcmd == PRIVATE_REMOVE) {
      if (!*buf)
			send_to_char(ch, "Who do you wish to add to you private channel?\r\n");
      else if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
			send_to_char(ch, NOPERSON);
      else if (IS_NPC(vict))
			send_to_char(ch, "NPC's cannot be on private channels\r\n");
      else if (GET_PRIVATE(vict) != GET_IDNUM(ch)) {
			send_to_char(ch,"%s is NOT on your Private Channel!\r\n",
							 GET_NAME(vict));
      } else {
			GET_PRIVATE(vict) = 0;
			send_to_char(ch, "You have been REMOVED from %s's Private "
							 "Channel!\r\n", GET_NAME(ch));
			send_to_char(vict, "%s has been REMOVED from your Private "
							 "Channel!\r\n", GET_NAME(vict));
      }
	} else if (subcmd == PRIVATE_ADD) {
		if (GET_PRIVATE(ch) != GET_IDNUM(ch))
			send_to_char(ch, "You must open your own private channel first\r\n");
		else if (!*buf)
			send_to_char(ch, "Who do you wish to add to you private channel?\r\n");
		else if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
			send_to_char(ch, NOPERSON);
		else if (ch == vict)
			GET_PRIVATE(ch) = GET_IDNUM(ch);
		else if (IS_NPC(vict))
			send_to_char(ch, "NPC's cannot be added to private channels\r\n");
		else if (GET_PRIVATE(vict) != 0) {
			send_to_char(ch, "%s is already on another private channel!\r\n",
							 GET_NAME(vict));
		} else {
			GET_PRIVATE(vict) = GET_IDNUM(ch);
			send_to_char(vict, "You have been ADDED to %s's Private Channel!\r\n",
							 GET_NAME(ch));
			send_to_char(ch, "%s has been ADDED to your Private Channel!\r\n",
							 GET_NAME(vict));
		}
	}
}


ACMD(do_talk)
{
	char arg[MAX_INPUT_LENGTH];
   struct char_data *mob;

   one_argument(argument, arg);

	mob = get_char_room_vis(ch, arg, NULL);
   if (!mob) {
      send_to_char(ch, "They don't seem to be here.\r\n");
      return;
	}

   if (!IS_NPC(mob))
      send_to_char(ch, "Why don't you just use say?\r\n");
   else if (!*arg)
      send_to_char(ch, "Talk to whom?\r\n");
   else if (GET_POS(mob) != POS_STANDING)
      send_to_char(ch, "Hmm, they aren't in the right position for that.\r\n");
   else
      mprog_talk_trigger(mob, ch);
}                                           
