/* ************************************************************************
 *   File: act.create.c					Part of CircleMUD *
 *  Usage: Player-level object creation stuff				  *
 *									  *
 *  All rights reserved.	 See license.doc for complete information.	  *
 *									  *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.		  *
 ************************************************************************ */

/* I rewrote substantial parts of this code.  In some places I'm just
 * simply less harsh up front, but more insidious (e.g. brew failure
 * means you mix up a poison potion rather then having it explode on
 * you).  In other places, I just durn didn't like how it was done.
 * Still, the bulk of the code, and certainly the structure and ideas
 * for this code, comes from someone else.  However, I have no idea
 * who that person is, since they never bothered to include a name or
 * contact info anywhere :p    Oh well....  -ejg */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"


/* struct for syls */
struct syllable {
	const char *org;
	const char *news;
};

/* extern variables */
extern struct spell_info_type spell_info[];
extern struct syllable syls[];


/* extern procedures */
int mag_manacost(struct char_data * ch, int spellnum);


/* location functions */
ACMD(do_brew);
ACMD(do_forge);
ACMD(do_scribe);


char *get_spell_name(char *argument)
{
	char *s;

	s = strtok(argument, "'");
	s = strtok(NULL, "'");
  
	return s;
}


char *potion_names[] = {
	"milky white",       /* 0 */
	"bubbling white",    /* 1 */
	"glowing ivory",
	"glowing blue",
	"bubbling yellow",
	"light green",       /* 5 */
	"gritty brown",
	"blood red",
	"swirling purple",
	"flickering green",
	"cloudy blue",       /* 10 */
	"glowing red",
	"sparkling white",
	"incandescent blue",
	"bright pink",
	"glowing beige",     /* 15 */
};

void make_potion(struct char_data *ch, int potion, struct obj_data *container)
{
	struct obj_data *final_potion;
	struct extra_descr_data *new_descr;
	int can_make = TRUE, mana, num = 0;
	int goal, roll, orig_potion = potion;
	char buf2[MAX_STRING_LENGTH] = "";
    
	/* Modify this list to suit which spells you want to be able to mix. */
	/* Done - I modified this list a bit.  -ejg */
	switch (potion) {
		case SPELL_BLESS:         num = 0; break;
		case SPELL_CURE_BLIND:    num = 1; break;
		case SPELL_CURE_LIGHT:    num = 2; break;
		case SPELL_CURE_CRITIC:   num = 3; break;
		case SPELL_DETECT_ALIGN:  num = 4; break;
		case SPELL_DETECT_INVIS:  num = 5; break;
		case SPELL_DETECT_MAGIC:  num = 6; break;
		case SPELL_FLY:           num = 7; break;
		case SPELL_HEAL:          num = 8; break;
		case SPELL_INFRAVISION:   num = 9; break;
		case SPELL_INVISIBLE:     num = 10; break;
		case SPELL_REMOVE_POISON: num = 11; break;
		case SPELL_SANCTUARY:     num = 12; break;
		case SPELL_SENSE_LIFE:    num = 13; break;
		case SPELL_STRENGTH:      num = 14; break;
		case SPELL_WATERWALK:     num = 15; break;
		default:
			can_make = FALSE;
			break;
	}

	mana = mag_manacost(ch, potion);

	/* Original code didn't do any checks against your skill level!
	 * *boggle*  -ejg */
	goal = GET_SKILL(ch, SKILL_BREW);
	goal += (IS_CLERIC(ch) ? GET_WIS(ch) : GET_INT(ch));
	goal += GET_LEVEL(ch);
	goal -= mana / 5;
	if (goal < 1)
		goal = 1; /* Always a minimal chance of success. */

	roll = rand_number(1, 101);
	if (GET_LEVEL(ch) >= LVL_IMMORT) /* Immorts never fail! */
		roll = 0;

	if (can_make == FALSE) {
		send_to_char(ch, "That spell cannot be mixed into a potion.\r\n");
		return;
	} else if (roll > goal) {
		if (roll >= 95) /* Minor failure - wrong type of potion! */
			potion = SPELL_POISON;
		else {
			send_to_char(ch, "You fail.\r\n");
			extract_obj(container);
			return;
		}
	}

	mana *= 3; /* costs 3x casting mana to brew */
	if (GET_LEVEL(ch) >= LVL_IMMORT)
		mana = 0;
	
	if (GET_MANA(ch) - mana > 0) {
		GET_MANA(ch) -= mana;
		send_to_char(ch, "You create a %s potion.\r\n", 
						 spell_info[orig_potion].name);
		act("$n creates a potion!", FALSE, ch, 0, 0, TO_ROOM);
		extract_obj(container);
	} else {
		send_to_char(ch, "You don't have enough mana to mix that potion!\r\n");
		return;
	}
	
	final_potion = create_obj();

	final_potion->item_number = NOTHING;
	IN_ROOM(final_potion) = NOWHERE;
	snprintf(buf2, sizeof(buf2), "%s %s potion", 
				potion_names[num], spell_info[orig_potion].name);
	final_potion->name = strdup(buf2);

	snprintf(buf2, sizeof(buf2), "A %s potion lies here.", potion_names[num]);
	final_potion->description = strdup(buf2);

	snprintf(buf2, sizeof(buf2), "a %s potion", potion_names[num]);
	final_potion->short_description = strdup(buf2);

	/* extra description coolness! */
	CREATE(new_descr, struct extra_descr_data, 1);
	new_descr->keyword = strdup(final_potion->name);
	snprintf(buf2, sizeof(buf2), "It appears to be a %s potion.", 
				spell_info[orig_potion].name);
	new_descr->description = strdup(buf2);
	new_descr->next = NULL;
	final_potion->ex_description = new_descr;
 
	GET_OBJ_TYPE(final_potion) = ITEM_POTION;
	GET_OBJ_WEAR(final_potion).ITEM_WEAR_TAKE = 1;
	GET_OBJ_EXTRA(final_potion).ITEM_NORENT = 1;
	GET_OBJ_VAL(final_potion, 0) = GET_LEVEL(ch);
	GET_OBJ_VAL(final_potion, 1) = potion;
	/* Good rolls means better potions!  -ejg */
	GET_OBJ_VAL(final_potion, 2) = (roll <= 5 ? potion : -1);
	GET_OBJ_VAL(final_potion, 3) = (roll <= 1 ? potion : -1);
	GET_OBJ_COST(final_potion) = GET_LEVEL(ch) * 50;
	GET_OBJ_WEIGHT(final_potion) = 1;
	GET_OBJ_RENT(final_potion) = 0;
		
	obj_to_char(final_potion, ch);
}

ACMD(do_brew)
{
	struct obj_data *container = NULL;
	struct obj_data *obj, *next_obj;
	char bottle_name[MAX_STRING_LENGTH];
	char spell_name[MAX_STRING_LENGTH];
	char *temp1, *temp2;
	int potion, found = FALSE;

	temp1 = one_argument(argument, bottle_name);
	
	/* sanity check */
	if (temp1) {
		temp2 = get_spell_name(temp1);
		if (temp2)
			strlcpy(spell_name, temp2, sizeof(spell_name));
	} else {
		bottle_name[0] = '\0';
		spell_name[0] = '\0';
	} 
	
     
	if (GET_SKILL(ch, SKILL_BREW) == 0) {
		send_to_char(ch, "You have no idea how to mix potions!\r\n");
		return;
	}

	if (!*bottle_name || !*spell_name) {
		send_to_char(ch, "What do you wish to mix in where?\r\n");
		return;
	}

	for (obj = ch->carrying; obj; obj = next_obj) {
		next_obj = obj->next_content;
		if (obj == NULL)
			return;
		else if (!(container = get_obj_in_list_vis(ch, bottle_name, NULL, ch->carrying))) 
			continue;
		else
			found = TRUE;
	}

	if (found != FALSE && (GET_OBJ_TYPE(container) != ITEM_DRINKCON)) {
		send_to_char(ch, "That item is not a drink container!\r\n");
		return;
	}

	if (found == FALSE) {
		send_to_char(ch, "You don't have %s in your inventory!\r\n", bottle_name);
		return;
	}

	if (!spell_name || !*spell_name) {
		send_to_char(ch, "Spell names must be enclosed in single quotes!\r\n");
		return;
	}
 
	potion = find_skill_num(spell_name);	

	if (potion < 1 || potion > MAX_SPELLS) {
		send_to_char(ch, "Mix what spell?!?\r\n");
		return;
	}

	if (GET_LEVEL(ch) < spell_info[potion].min_level[(int) GET_CLASS(ch)]) {
		send_to_char(ch, "You do not know how to make that potion!\r\n");
		return;
	}

	make_potion(ch, potion, container);
}


/* Blah - this (spell garbling) changed quite a bit since the patch
 * level this module was originally written for.  Hope I haven't
 * created any memory leaks here...  -ejg */
char *garble_spell(int spellnum)
{
	char lbuf[256];
	int j, ofs = 0;
	char buf[MAX_STRING_LENGTH];
	*buf = '\0';

	/* This is basically just copied from say_spell() in spell_parser.c  -ejg */
	strlcpy(lbuf, skill_name(spellnum), sizeof(lbuf));

	while (lbuf[ofs]) {
		for (j = 0; *(syls[j].org); j++) {
			if (!strncmp(syls[j].org, lbuf + ofs, strlen(syls[j].org))) {
				strcat(buf, syls[j].news);	/* strcat: BAD */
				ofs += strlen(syls[j].org);
				break;
			}
		}
		/* i.e., we didn't find a match in syls[] */
		if (!*syls[j].org) {
			log("No entry in syllable table for substring of '%s'", lbuf);
			ofs++;
		}
	}

	return strdup(buf);
}


void make_scroll(struct char_data *ch, int scroll, struct obj_data *paper)
{
	struct obj_data *final_scroll;
	struct extra_descr_data *new_descr;
	int can_make = TRUE, mana;
	int roll, goal, orig_scroll = scroll;
	char buf2[MAX_STRING_LENGTH] = "";
	char *buf = NULL;

	/* add a case statement here for prohibited spells */

 
	mana = mag_manacost(ch, scroll);

	/* Original code didn't do any checks against your skill level!
	 * *boggle*  -ejg */
	goal = GET_SKILL(ch, SKILL_SCRIBE);
	goal += (IS_CLERIC(ch) ? GET_WIS(ch) : GET_INT(ch));
	goal -= mana / 5;
	if (goal < 1)
		goal = 1; /* Always a minimal chance of success. */

	roll = rand_number(1, 101);
	if (GET_LEVEL(ch) >= LVL_IMMORT) /* Immorts never fail! */
		roll = 0;

	if (can_make == FALSE) {
		send_to_char(ch, "That spell cannot be scribed into a scroll.\r\n");
		return;
	} else if (roll > goal) {
		if (roll >= 95)
			scroll = SPELL_CURSE;
		else {
			send_to_char(ch, "You fail.\r\n");
			extract_obj(paper);
			return;
		}
	}

	mana *= 5; /* 5x the casting cost to scribe */
	if (GET_LEVEL(ch) >= LVL_IMMORT)
		mana = 0;

	if (GET_MANA(ch) - mana > 0) {
		GET_MANA(ch) -= mana;
		send_to_char(ch, "You create a scroll of %s.\r\n",
						 spell_info[orig_scroll].name);
		act("$n creates a scroll!", FALSE, ch, 0, 0, TO_ROOM);
		extract_obj(paper);
	} else {
		send_to_char(ch, "You don't have enough mana to scribe such"
						 " a powerful spell!\r\n");
		return;
	}
	
	final_scroll = create_obj();

	final_scroll->item_number = NOTHING;
	IN_ROOM(final_scroll) = NOWHERE;
	
	buf = garble_spell(orig_scroll);
	snprintf(buf2, sizeof(buf2), "%s %s scroll", 
				spell_info[orig_scroll].name, buf);
	final_scroll->name = strdup(buf2);

	snprintf(buf2, sizeof(buf2), 
				"Some parchment inscribed with the runes '%s' lies here.", buf);
	final_scroll->description = strdup(buf2);

	snprintf(buf2, sizeof(buf2), "a %s scroll", buf);
	final_scroll->short_description = strdup(buf2);

	/* extra description coolness! */
	CREATE(new_descr, struct extra_descr_data, 1);
	new_descr->keyword = strdup(final_scroll->name);
	snprintf(buf2, sizeof(buf2), "It appears to be a %s scroll.", 
				spell_info[orig_scroll].name);
	new_descr->description = strdup(buf2);
	new_descr->next = NULL;
	final_scroll->ex_description = new_descr;
 
	GET_OBJ_TYPE(final_scroll) = ITEM_SCROLL;
	GET_OBJ_WEAR(final_scroll).ITEM_WEAR_TAKE = 1;
	GET_OBJ_EXTRA(final_scroll).ITEM_NORENT = 1;
	GET_OBJ_VAL(final_scroll, 0) = GET_LEVEL(ch);
	GET_OBJ_VAL(final_scroll, 1) = scroll;
	/* Extra coolness for good rolls!  -ejg */
	GET_OBJ_VAL(final_scroll, 2) = (roll <= 5 ? scroll : -1);
	GET_OBJ_VAL(final_scroll, 3) = (roll <= 1 ? scroll : -1);
	GET_OBJ_COST(final_scroll) = GET_LEVEL(ch) * 300;
	GET_OBJ_WEIGHT(final_scroll) = 1;
	GET_OBJ_RENT(final_scroll) = 0;
	
	free(buf);
	obj_to_char(final_scroll, ch);
}


ACMD(do_scribe)
{
	struct obj_data *paper = NULL;
	struct obj_data *obj, *next_obj;
	char paper_name[MAX_STRING_LENGTH];
	char spell_name[MAX_STRING_LENGTH];
	char *temp1, *temp2;
	int scroll = 0, found = FALSE;

	temp1 = one_argument(argument, paper_name);

	/* sanity check */
	if (temp1) {
		temp2 = get_spell_name(temp1);
		if (temp2)
			strlcpy(spell_name, temp2, sizeof(spell_name));
	} else {
		paper_name[0] = '\0';
		spell_name[0] = '\0';
	}


	if (GET_SKILL(ch, SKILL_SCRIBE) == 0) {
		send_to_char(ch, "You have no idea how to scribe scrolls!\r\n");
		return;
	}

	if (!*paper_name || !*spell_name) {
		send_to_char(ch, "What do you wish to scribe where?\r\n");
		return;
	}

	for (obj = ch->carrying; obj; obj = next_obj) {
		next_obj = obj->next_content;
		if (obj == NULL)
			return;
		else if (!(paper = get_obj_in_list_vis(ch, paper_name, NULL, ch->carrying))) 
			continue;
		else
			found = TRUE;
	}

	if (found && (GET_OBJ_TYPE(paper) != ITEM_NOTE)) {
		send_to_char(ch, "You can't write on that!\r\n");
		return;
	}

	if (found == FALSE) {
		send_to_char(ch, "You don't have %s in your inventory!\r\n", paper_name);
		return;
	}

	if (!spell_name || !*spell_name) {
		send_to_char(ch, "Spell names must be enclosed in single quotes!\r\n");
		return;
	} 

	scroll = find_skill_num(spell_name);	

	if (scroll < 1 || scroll > MAX_SPELLS) {
		send_to_char(ch, "Scribe what spell?!?\r\n");
		return;
	}

	if (GET_LEVEL(ch) < spell_info[scroll].min_level[(int) GET_CLASS(ch)]) {
		send_to_char(ch, "You are not schooled enough to cast that spell!\r\n");
		return;
	}

	make_scroll(ch, scroll, paper);
}


ACMD(do_forge)
{
	struct obj_data *weapon = NULL;
	struct obj_data *obj, *next_obj;
	char weapon_name[MAX_STRING_LENGTH];
	int found = FALSE, goal = 0, roll = 0, dam = 0;
	int i, hit_mod = 0, dam_mod = 0;

	one_argument(argument, weapon_name);

	if (GET_SKILL(ch, SKILL_FORGE) == 0) {
		send_to_char(ch, "You have no idea how to forge weapons!\r\n");
		return;
	}

	if (!*weapon_name) {
		send_to_char(ch, "What do you wish to forge?\r\n");
		return;
	}

	for (obj = ch->carrying; obj; obj = next_obj) {
		next_obj = obj->next_content;
		if (obj == NULL)
			return;
		else if (!(weapon = get_obj_in_list_vis(ch, weapon_name, NULL,
															 ch->carrying))) 
			continue;
		else
			found = TRUE;
	}
	
	if (found == FALSE) {
		send_to_char(ch, "You don't have %s in your inventory!\r\n", weapon_name);
		return;
	}

	if (found && (GET_OBJ_TYPE(weapon) != ITEM_WEAPON)) {
		send_to_char(ch, "It doesn't look like %s would make a"
						 " good weapon...\r\n", weapon_name);
		return;
	}

	if (GET_OBJ_VAL(weapon, 0) > 0 && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "You cannot forge a weapon more than once!\r\n");
		return;
	}

	if (OBJ_FLAGGED(weapon, ITEM_MAGIC)) {
		send_to_char(ch, "The weapon is imbued with magical powers beyond your "
						 "grasp.\r\nYou cannot further affect its form.\r\n");
		return;
	}

	/* determine success probability */
	goal = GET_SKILL(ch, SKILL_FORGE);
	goal += (GET_STR(ch) / 2);
	goal += (GET_DEX(ch) / 2);
	goal += (GET_INT(ch) / 3);
	/* Minus the max damage of the weapon */
	dam = (GET_OBJ_VAL(weapon, 1) * GET_OBJ_VAL(weapon, 2));
	goal -= dam;

	roll = (GET_LEVEL(ch) >= LVL_IMMORT ? 0 : rand_number(1, 101));

	/* Figure out what happened and assign bonuses/penalties accordingly */
	if (roll > goal) {
		/* Minor failure */
		hit_mod = -1;
		
		if (roll >= 100) {
			/* Catastrophic failure!  Weapon destroyed... */
			send_to_char(ch, "As you pound out the dents in the weapon, "
							 "you hit a weak spot and it shatters!\r\n");
			act("$n tries to forge a weapon, but it shatters!", 
				 FALSE, ch, 0,0, TO_ROOM);

			extract_obj(weapon);
			GET_HIT(ch) -= dam / 2;
			update_pos(ch);
			return;
		} else if (roll >= 95) { /* Major failure */
			dam_mod = -1;
		} else {
			/* We're already incurring a -1 hit roll modifier for
			 * failure, that's enough in this case.  -ejg */
		}
	} else {
		/* Minor success */
		hit_mod = 1;
		
		if (roll <= 5) /* Major success */
			dam_mod = 1;

		if (roll <= 1) /* Fantastic success */
		{
			hit_mod = 2;
			dam_mod = 2;
		}
	}
	
	GET_OBJ_VAL(weapon, 0) = GET_LEVEL(ch);

	/* Now actually change the weapon... */
	for (i = 0; i < MAX_OBJ_AFFECT; i++) {
		if (weapon->affected[i].location == APPLY_NONE) {
			if (hit_mod && !dam_mod) {
				weapon->affected[i].location = APPLY_HITROLL;
				weapon->affected[i].modifier = hit_mod;
			} else if (hit_mod && dam_mod) {
				weapon->affected[i].location = APPLY_HITNDAM;
				weapon->affected[i].modifier = hit_mod;
			}
			break;
		}
	}

	if (i == MAX_OBJ_AFFECT) {
		send_to_char(ch, "This weapon is too powerful to be forged!\r\n");
		return;
	}
	
	/* Change the value of the weapon */
	if (hit_mod < 0) {
		if (dam_mod < 0)
			GET_OBJ_RENT(weapon) /= 10;
		else
			GET_OBJ_RENT(weapon) /= 5;
	} else if (hit_mod > 0) {
		if (dam_mod > 0)
			GET_OBJ_RENT(weapon) *= 5;
		else
			GET_OBJ_RENT(weapon) *= 2.5;
	}

	send_to_char(ch, "You have forged new life into the weapon!\r\n");
	act("$n vigorously pounds on a weapon!", FALSE, ch, 0, 0, TO_ROOM);
}

