#include "info.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "screen.h"


void do_auto_exits(struct char_data *ch)
{
	int door;
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	*buf = '\0';

	if (AFF_FLAGGED(ch, AFF_BLIND)) {
		send_to_char(ch, "You can't see a damned thing, you're blind!\r\n");
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	for (door = 0; door < NUM_OF_DIRS; door++) {
		if (EXIT(ch, door) && EXIT(ch, door)->to_room != NOWHERE &&
			 !EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED)) {
			if (GET_LEVEL(ch) >= LVL_IMMORT)
				snprintf(buf2, sizeof(buf2), "%-5s - [%5d] %s\r\n", dirs[door],
							world[EXIT(ch, door)->to_room].number,
							world[EXIT(ch, door)->to_room].name);
			else {
				snprintf(buf2, sizeof(buf2), "%-5s - ", dirs[door]);
				if (IS_DARK(EXIT(ch, door)->to_room) && !CAN_SEE_IN_DARK(ch))
					strcat(buf2, "Too dark to tell\r\n");
				else {
					strcat(buf2, world[EXIT(ch, door)->to_room].name);
					strcat(buf2, "\r\n");
				}
			}
			strcat(buf, CAP(buf2));
		} else if (EXIT(ch, door) && EXIT(ch, door)->to_room != NOWHERE &&
					  EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED)) {
			if (GET_LEVEL(ch) >= LVL_IMMORT) {
				snprintf(buf2, sizeof(buf2), "%-5s - [%5d] %s - CLOSED", 
							dirs[door], world[EXIT(ch, door)->to_room].number,
							world[EXIT(ch, door)->to_room].name);
				if (EXIT_FLAGGED(EXIT(ch, door), EX_LOCKED))
					strcat(buf2, " - LOCKED");
				if (EXIT_FLAGGED(EXIT(ch, door), EX_PICKPROOF))
					strcat(buf2, " - PICKPROOF");
				if (EXIT_FLAGGED(EXIT(ch, door), EX_HIDDEN))
					strcat(buf2, " - HIDDEN");
				strcat(buf2, "\r\n");
			} else
				snprintf(buf2, sizeof(buf2), "%-5s - The %s is closed\r\n", 
							dirs[door], fname(EXIT(ch, door)->keyword));
			strcat(buf, CAP(buf2));
		}
	}
	send_to_char(ch, "Obvious exits:\r\n");

	if (*buf)
		send_to_char(ch, buf);
	else
		send_to_char(ch, " None.\r\n");
}


/* The original do_auto_exits() function for those who prefer the
 * basic stock behavior for autoexit: 
void do_auto_exits(struct char_data *ch)
{
	int door, slen = 0;

	send_to_char(ch, "%s[ Exits: ", CCCYN(ch, C_NRM));

	for (door = 0; door < NUM_OF_DIRS; door++) {
		if (!EXIT(ch, door) || EXIT(ch, door)->to_room == NOWHERE)
			continue;
		if (EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED))
			continue;

		send_to_char(ch, "%c ", LOWER(*dirs[door]));
		slen++;
	}

	send_to_char(ch, "%s]%s\r\n", slen ? "" : "None!", CCNRM(ch, C_NRM));
}
*/


ACMD(do_exits)
{
	int door, len = 0;

	if (AFF_FLAGGED(ch, AFF_BLIND)) {
		send_to_char(ch, "You can't see a damned thing, you're blind!\r\n");
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	send_to_char(ch, "Obvious exits:\r\n");

	for (door = 0; door < NUM_OF_DIRS; door++) {
		if (!EXIT(ch, door) || EXIT(ch, door)->to_room == NOWHERE)
			continue;
		if (EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED))
			continue;
		if (EXIT_FLAGGED(EXIT(ch, door), EX_HIDDEN))
			continue;

		len++;

		if (GET_LEVEL(ch) >= LVL_IMMORT)
			send_to_char(ch, "%-5s - [%5d] %s\r\n", 
							 dirs[door], GET_ROOM_VNUM(EXIT(ch, door)->to_room),
							 world[EXIT(ch, door)->to_room].name);
		else
			send_to_char(ch, "%-5s - %s\r\n", 
							 dirs[door], IS_DARK(EXIT(ch, door)->to_room) &&
							 !CAN_SEE_IN_DARK(ch) ? "Too dark to tell." : world[EXIT(ch, door)->to_room].name);
	}

	if (!len)
		send_to_char(ch, " None.\r\n");
}


char *blood_messages[] = {
	"Should never see this.",
	"There's a little blood here.",
	"You're standing in some blood.",
	"The blood in this room is flowing.",
	"There's so much blood here it's intoxicating!",
	"How much more blood can there be in any one room?",
	"Such carnage. The God of Death is feastin' tonight!",
	"You are splashing around in the blood of the slain!",
	"Even the walls are revolted by the death and destruction!",
	"The Gods should show some mercy and cleanse this horrid room!",
	"So much blood has been shed here, you are drowning in it!",
	"\n"
};


void look_at_room(struct char_data *ch, int ignore_brief)
{
	if (!ch->desc)
		return;

	if (IS_DARK(IN_ROOM(ch)) && !CAN_SEE_IN_DARK(ch)) {
		send_to_char(ch, "It is pitch black...\r\n");
		return;
	} else if (AFF_FLAGGED(ch, AFF_BLIND)) {
		send_to_char(ch, "You see nothing but infinite darkness...\r\n");
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	send_to_char(ch, "%s", CCCYN(ch, C_NRM));
	if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_ROOMFLAGS)) {
		char buf[MAX_STRING_LENGTH];

		sprintfield((char*)&ROOM_FLAGS(IN_ROOM(ch)), room_bits, buf, sizeof(buf));
		send_to_char(ch, "[%5d] %s [ %s]", GET_ROOM_VNUM(IN_ROOM(ch)), world[IN_ROOM(ch)].name, buf);
	} else
		send_to_char(ch, world[IN_ROOM(ch)].name);

	send_to_char(ch, "%s\r\n", CCNRM(ch, C_NRM));

	if ((!IS_NPC(ch) && !PRF_FLAGGED(ch, PRF_BRIEF)) || ignore_brief ||
		 ROOM_FLAGGED(IN_ROOM(ch), ROOM_DEATH))
		send_to_char(ch, world[IN_ROOM(ch)].description);

	/* autoexits */
	if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_AUTOEXIT))
		do_auto_exits(ch);

	/* now list characters & objects */
	send_to_char(ch, "%s", CCGRN(ch, C_NRM));
	list_obj_to_char(world[IN_ROOM(ch)].contents, ch, SHOW_OBJ_LONG, FALSE);
	send_to_char(ch, "%s", CCYEL(ch, C_NRM));
	list_char_to_char(world[IN_ROOM(ch)].people, ch);
	send_to_char(ch, "%s", CCNRM(ch, C_NRM));

	if (RM_BLOOD(IN_ROOM(ch)) > 0)
		act(blood_messages[RM_BLOOD(IN_ROOM(ch))], FALSE, ch, 0, 0, TO_CHAR);
}


void look_in_direction(struct char_data *ch, int dir)
{
	if (EXIT(ch, dir)) {
		if (EXIT(ch, dir)->general_description)
			send_to_char(ch, "%s", EXIT(ch, dir)->general_description);
		else
			send_to_char(ch, "You see nothing special.\r\n");

		if (EXIT_FLAGGED(EXIT(ch, dir), EX_CLOSED) && EXIT(ch, dir)->keyword)
			send_to_char(ch, "The %s is closed.\r\n", fname(EXIT(ch, dir)->keyword));
		else if (EXIT_FLAGGED(EXIT(ch, dir), EX_ISDOOR) && EXIT(ch, dir)->keyword)
			send_to_char(ch, "The %s is open.\r\n", fname(EXIT(ch, dir)->keyword));
	} else
		send_to_char(ch, "Nothing special there...\r\n");
}


/* 
	just add this file to the end of act.informative.c (MAKE SURE YOU PRESERVE 
	THE SPACING ON THE MACRO DEFINES), and add two things to interpreter.c:
	a) a prototype for do_scan - put the line
	ACMD(do_scan);
	somewhere near the top, with all the other ACMD prototypes.
	b) put an entry in the command list for 'scan', I suggest after 'score'.

	you can contact me sjmaster@mit.edu if you have problems, which you should
	not.
*/

void list_scanned_chars(struct char_data * list, struct char_data * ch, int distance, int door)
{
	const char *how_far[] = {
		"close by",
		"a ways off",
		"far off to the"
	};

	struct char_data *i;
	int count = 0;
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	*buf = '\0';

	/* this loop is a quick, easy way to help make a grammatical
	 * sentence (i.e., "You see x, x, y, and z." with commas, "and",
	 * etc.) */

	for (i = list; i; i = i->next_in_room)

		/* put any other conditions for scanning someone in this if
			statement - i.e., if (CAN_SEE(ch, i) && condition2 &&
			condition3) or whatever */

		if (CAN_SEE(ch, i))
			count++;

	if (!count)
		return;

	for (i = list; i; i = i->next_in_room) {

		/* make sure to add changes to the if statement above to this
			one also, using or's to join them.. i.e., if (!CAN_SEE(ch, i)
			|| !condition2 || !condition3) */

		if (!CAN_SEE(ch, i))
			continue; 

		if (!*buf)
			sprintf(buf, "You see %s", GET_NAME(i));
		else 
			sprintf(buf + strlen(buf), "%s", GET_NAME(i));

		if (--count > 1)
			strcat(buf, ", ");
		else if (count == 1)
			strcat(buf, " and ");
		else {
			snprintf(buf2, sizeof(buf2), " %s %s.\r\n", 
						how_far[distance], dirs[door]);
			strcat(buf, buf2);      
		}
    
	}
	send_to_char(ch, buf);
}


/* utils.h: #define EXIT(ch, door)  (world[IN_ROOM(ch)].dir_option[door]) */
#define _2ND_EXIT(ch, door) (world[EXIT(ch, door)->to_room].dir_option[door])
#define _3RD_EXIT(ch, door) (world[_2ND_EXIT(ch, door)->to_room].dir_option[door])

ACMD(do_scan)
{
	/* >scan
		You quickly scan the area.
		You see John, a large horse and Frank close by north.
		You see a small rabbit a ways off south.
		You see a huge dragon and a griffon far off to the west.

	*/
	int door;
	char buf[MAX_STRING_LENGTH];
	
	*buf = '\0';
  
	if (AFF_FLAGGED(ch, AFF_BLIND)) {
		send_to_char(ch, "You can't see a damned thing, you're blind!\r\n");
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	/* may want to add more restrictions here, too */
	send_to_char(ch, "You quickly scan the area.\r\n");
	for (door = 0; door < NUM_OF_DIRS - 2; door++) { /* don't scan up/down */
		if (EXIT(ch, door) && EXIT(ch, door)->to_room != NOWHERE &&
			 !EXIT_FLAGGED(EXIT(ch, door), EX_CLOSED) && 
			 !IS_DARK(EXIT(ch, door)->to_room)) {
			if (world[EXIT(ch, door)->to_room].people)
				list_scanned_chars(world[EXIT(ch, door)->to_room].people,
										 ch, 0, door);
			else if (_2ND_EXIT(ch, door) &&
						_2ND_EXIT(ch, door)->to_room !=  NOWHERE && 
						!EXIT_FLAGGED(_2ND_EXIT(ch, door), EX_CLOSED) && 
						!IS_DARK(_2ND_EXIT(ch, door)->to_room)) {
				/* check the second room away */
				if (world[_2ND_EXIT(ch, door)->to_room].people) 
					list_scanned_chars(world[_2ND_EXIT(ch, door)->to_room].people,
											 ch, 1, door);
				else if (_3RD_EXIT(ch, door) && 
							_3RD_EXIT(ch, door)->to_room != NOWHERE && 
							!EXIT_FLAGGED(_3RD_EXIT(ch, door), EX_CLOSED) && 
							!IS_DARK(_3RD_EXIT(ch, door)->to_room)) {
					/* check the third room */
					if (world[_3RD_EXIT(ch, door)->to_room].people) 
						list_scanned_chars(world[_3RD_EXIT(ch, door)->to_room].people,
												 ch, 2, door);
				}
			}
		}             
	}
}

