#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"


/* External veriables */
extern int pt_allowed;

/* External functions */
SPECIAL(shop_keeper);
ACMD(do_scan);


/* Local functions */
ACMD(do_backstab);
ACMD(do_hide);
ACMD(do_poison);
ACMD(do_sap);
ACMD(do_sneak);
ACMD(do_steal);


ACMD(do_backstab)
{
	char buf[MAX_INPUT_LENGTH];
	struct char_data *vict;
	int roll, goal;

	if (IS_NPC(ch) || !GET_SKILL(ch, SKILL_BACKSTAB)) {
		send_to_char(ch, "You have no idea how to do that.\r\n");
		return;
	}

	one_argument(argument, buf);

	if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "Backstab who?\r\n");
		return;
	}

	if (vict == ch) {
		send_to_char(ch, "How can you sneak up on yourself?\r\n");
		return;
	}

	if (!GET_EQ(ch, WEAR_WIELD)) {
		send_to_char(ch, "You need to wield a weapon to make it a success.\r\n");
		return;
	}

	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_PIERCE - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_STING - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_STAB - TYPE_HIT) {
		send_to_char(ch, "Only piercing weapons can be used for "
						 "backstabbing.\r\n");
		return;
	}
	if (FIGHTING(vict)) {
		send_to_char(ch, "You can't backstab a fighting person -- they're "
						 "too alert!\r\n");
		return;
	}

	if (!AFF_FLAGGED(ch, AFF_SNEAK) ||
		 ((MOB_FLAGGED(vict, MOB_AWARE) || AFF_FLAGGED(vict, AFF_AWARE)) && 
		  AWAKE(vict))) {
		act("You notice $N lunging at you!", FALSE, vict, 0, ch, TO_CHAR);
		act("$e notices you lunging at $m!", FALSE, vict, 0, ch, TO_VICT);
		act("$n notices $N lunging at $m!", FALSE, vict, 0, ch, TO_NOTVICT);
		hit(vict, ch, TYPE_UNDEFINED);
		return;
	}

	roll = rand_number(1, 101);	/* 101% is a complete failure */
	goal = GET_SKILL(ch, SKILL_BACKSTAB);

	if (AWAKE(vict) && (roll > goal))
		damage(ch, vict, 0, SKILL_BACKSTAB);
	else
		hit(ch, vict, SKILL_BACKSTAB);

	WAIT_STATE(ch, 2 * PULSE_VIOLENCE);
}


ACMD(do_sneak)
{
	struct affected_type af;
	byte roll = rand_number(1, 101);
	int goal = GET_SKILL(ch, SKILL_SNEAK);
	goal += dex_app_skill[GET_DEX(ch)].sneak;
	goal += race_app_skill[GET_RACE(ch)].sneak;

	if (IS_NPC(ch) || !GET_SKILL(ch, SKILL_SNEAK)) {
		send_to_char(ch, "You have no idea how to do that.\r\n");
		return;
	}
	send_to_char(ch, "Okay, you'll try to move silently for a while.\r\n");
	if (AFF_FLAGGED(ch, AFF_SNEAK))
		affect_from_char(ch, SKILL_SNEAK);

	if (roll > goal)
		return;

	af.type = SKILL_SNEAK;
	af.duration = GET_LEVEL(ch);
	af.modifier = 0;
	af.location = APPLY_NONE;
	af.bitvector.AFF_SNEAK = 1;
	affect_to_char(ch, &af);
}



ACMD(do_hide)
{
	byte roll = rand_number(1, 101);
	int goal = GET_SKILL(ch, SKILL_HIDE);
	goal += dex_app_skill[GET_DEX(ch)].hide;
	goal += race_app_skill[GET_RACE(ch)].hide;

	if (IS_NPC(ch) || !GET_SKILL(ch, SKILL_HIDE)) {
		send_to_char(ch, "You have no idea how to do that.\r\n");
		return;
	}

	send_to_char(ch, "You attempt to hide yourself.\r\n");

	if (AFF_FLAGGED(ch, AFF_HIDE))
		AFF_FLAGS(ch).AFF_HIDE = 0;

	if (roll > goal)
		return;

	AFF_FLAGS(ch).AFF_HIDE = 1;
}


ACMD(do_steal)
{
	struct char_data *vict;
	struct obj_data *obj;
	char vict_name[MAX_INPUT_LENGTH], obj_name[MAX_INPUT_LENGTH];
	char buf[MAX_STRING_LENGTH];
	int gold, eq_pos, pcsteal = 0, ohoh = 0, type = 0;
	int roll = rand_number(1, 101);
	roll -= dex_app_skill[GET_DEX(ch)].p_pocket;
	roll -= race_app_skill[GET_RACE(ch)].p_pocket;

	if (IS_NPC(ch) || !GET_SKILL(ch, SKILL_STEAL)) {
		send_to_char(ch, "You have no idea how to do that.\r\n");
		return;
	}

	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_PEACEFUL)) {
		send_to_char(ch, "This room just has such a peaceful, easy "
						 "feeling...\r\n");
		return;
	}

	two_arguments(argument, obj_name, vict_name);

	if (!(vict = get_char_vis(ch, vict_name, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "Steal what from who?\r\n");
		return;
	} else if (vict == ch) {
		send_to_char(ch, "Come on now, that's rather stupid!\r\n");
		return;
	}

	if (GET_POS(vict) < POS_SLEEPING)
		roll = -1;		/* ALWAYS SUCCESS, unless heavy object. */

	if (!pt_allowed && !IS_NPC(vict))
		pcsteal = 1;

	if (!AWAKE(vict))	/* Easier to steal from sleeping people. */
		roll -= 50;

	if (!AFF_FLAGGED(ch, AFF_SNEAK)) /* Harder if you aren't sneaking */
		roll += 25; 

	/* NO NO With Imp's and Shopkeepers, and if player thieving is not
	 * allowed */
	if (GET_LEVEL(vict) >= LVL_IMMORT || pcsteal ||
		 GET_MOB_SPEC(vict) == shop_keeper)
		roll = 101;		/* Failure */

	if (str_cmp(obj_name, "platinum") && is_abbrev(obj_name, "platinum") &&
		 str_cmp(obj_name, "gold") && is_abbrev(obj_name, "gold") &&
		 str_cmp(obj_name, "silver") && is_abbrev(obj_name, "silver") &&
		 str_cmp(obj_name, "copper") && is_abbrev(obj_name, "copper") &&
		 str_cmp(obj_name, "coins") && is_abbrev(obj_name, "coins")) {
		if (!(obj = get_obj_in_list_vis(ch, obj_name, NULL, vict->carrying))) {
			for (eq_pos = 0; eq_pos < NUM_WEARS; eq_pos++)
				if (GET_EQ(vict, eq_pos) &&
					 (isname(obj_name, GET_EQ(vict, eq_pos)->name)) &&
					 CAN_SEE_OBJ(ch, GET_EQ(vict, eq_pos))) {
					obj = GET_EQ(vict, eq_pos);
					break;
				}
			if (!obj) {
				act("$E hasn't got that item.", FALSE, ch, 0, vict, TO_CHAR);
				return;
			} else {			/* It is equipment */
				if ((GET_POS(vict) > POS_STUNNED)) {
					send_to_char(ch, "Steal the equipment now?  Impossible!\r\n");
					return;
				} else {
					act("You unequip $p and steal it.", FALSE, ch, obj, 0, TO_CHAR);
					act("$n steals $p from $N.", FALSE, ch, obj, vict, TO_NOTVICT);
					obj_to_char(unequip_char(vict, eq_pos), ch);
				}
			}
		} else {			/* obj found in inventory */
			roll += GET_OBJ_WEIGHT(obj);	/* Make heavy harder */

			if (roll > GET_SKILL(ch, SKILL_STEAL)) {
				ohoh = TRUE;
				send_to_char(ch, "Oops..\r\n");
				act("$n tried to steal something from you!", FALSE, ch, 0, vict, TO_VICT);
				act("$n tries to steal something from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
			} else {			/* Steal the item */
				if (IS_CARRYING_N(ch) + 1 < CAN_CARRY_N(ch)) {
					if (IS_CARRYING_W(ch) + GET_OBJ_WEIGHT(obj) < CAN_CARRY_W(ch)) {
						obj_from_char(obj);
						obj_to_char(obj, ch);
						send_to_char(ch, "Got it!\r\n");
					}
				} else
					send_to_char(ch, "You cannot carry that much.\r\n");
			}
		}
	} else {			/* Steal some coins */
		if (AWAKE(vict) && (roll > GET_SKILL(ch, SKILL_STEAL))) {
			ohoh = TRUE;
			send_to_char(ch, "Oops..\r\n");
			act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, vict, TO_VICT);
			act("$n tries to steal coins from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
		} else {
			/* Steal some gold coins */
			if (!strcmp(obj_name, "platinum") || is_abbrev(obj_name, "platinum")) 
				type = PLAT_COINS;
			else if (!strcmp(obj_name, "gold")  || is_abbrev(obj_name, "gold"))
				type = GOLD_COINS;
			else if (!strcmp(obj_name, "silver") || is_abbrev(obj_name, "silver"))
				type = SILVER_COINS;
			else if (!strcmp(obj_name, "copper") || is_abbrev(obj_name, "copper"))
				type = COPPER_COINS;
			else
				type = rand_number(0,3);

			/* Steal some gold coins */
			gold = (GET_MONEY(vict, type) * rand_number(1, 10)) / 100;
			gold = MIN(1782, gold);

			if (gold > 0) {
				if (type < 4 ) {
					remove_money_from_char(vict, gold, type);    
					add_money_to_char(ch, gold, type);
				} else {
					log("SYSERR: unset coin type passed through steal");  
					send_to_char(ch, "Error is do_steal - please report!\r\n");
					return;
				}

				if (gold > 1) {
					snprintf(buf, sizeof(buf), "Bingo!  You got %d %s coins "
								"from $N.", gold, coin_types[type]);
					act(buf, TRUE, ch, 0, vict, TO_CHAR);
					WAIT_STATE(ch, PULSE_VIOLENCE);
				} else {
					snprintf(buf, sizeof(buf), "You manage to swipe a solitary "
								"%s coin from $N.");
					act(buf, TRUE, ch, 0, vict, TO_CHAR);
					WAIT_STATE(ch, PULSE_VIOLENCE);
				}
			} else {
				snprintf(buf, sizeof(buf), "You couldn't get any %s from $N.", 
							coin_types[type]);
				act(buf, TRUE, ch, 0, vict, TO_CHAR);			 
				WAIT_STATE(ch, PULSE_VIOLENCE);
			}
		}
	}

	if (ohoh && IS_NPC(vict) && AWAKE(vict))
		hit(vict, ch, TYPE_UNDEFINED);
}


/* A spy skill written by Eric V. Bahmer Raptor of MultiMUD and world
   of pain.

   Basically all you need to do is put this in act.other.c and fill in
   the acmd protos in interpreter.c, define the skill in spells.h and
   assign it to a class in class.c, put it in spell_parser.c next to
   the other skills

   I wrote this when i got bored one day feel free to use it but keep
   my name in the comment block before the actual code. far as I know
   there shouldn't be any bugs.
*/
/* I renamed this skill from do_spy to do_scout  -ejg */
ACMD(do_scout)
{
   int roll, goal, scout_type, return_room;
   char *scout_dirs[] = {
		"north",
		"east",
		"south",
		"west",
		"up",
		"down",
		"\n"
   };

   /* 101% is a complete failure */
   roll = rand_number(1, 101);
   goal = GET_SKILL(ch, SKILL_SCOUT);

   scout_type = search_block(argument + 1, scout_dirs, FALSE);

	if (scout_type < 0 ||
		 !EXIT(ch, scout_type) || 
		 EXIT(ch, scout_type)->to_room == NOWHERE) {
		send_to_char(ch, "Scout where?\r\n");
		return;
	} else {
		if (GET_MOVE(ch) < 5)
			send_to_char(ch, "You are too tired to scout anywhere.\r\n");
		else {
			if (roll > goal) {
				send_to_char(ch, "You failed to uncover any information.\r\n");
				GET_MOVE(ch) = MAX(0, MIN(GET_MAX_MOVE(ch), GET_MOVE(ch) - 2));
			} else {
				if (EXIT_FLAGGED(EXIT(ch, scout_type), EX_CLOSED) && 
					 EXIT(ch, scout_type)->keyword) {
					send_to_char(ch, "The %s is closed.\r\n",
									 fname(EXIT(ch, scout_type)->keyword));
					GET_MOVE(ch) = MAX(0, MIN(GET_MAX_MOVE(ch), GET_MOVE(ch) - 2));
				} else {
					GET_MOVE(ch) = MAX(0, MIN(GET_MAX_MOVE(ch), GET_MOVE(ch) - 5));
					return_room = IN_ROOM(ch);
					char_from_room(ch);
					char_to_room(ch, world[return_room].dir_option[scout_type]->to_room);
					send_to_char(ch, "You scout around a bit and find:\r\n\r\n");
					look_at_room(ch, 1);
					char_from_room(ch);
					char_to_room(ch, return_room);
					act("$n scouts about for a while.", TRUE, ch, 0, 0, TO_NOTVICT);
				}
			}
		}
	}
}



#define CAN_LISTEN_BEHIND_DOOR(ch,dir)  \
                ((EXIT(ch, dir) && EXIT(ch, dir)->to_room != NOWHERE && \
                 EXIT_FLAGGED(EXIT(ch, dir), EX_CLOSED)))

ACMD(do_listen)
{
   struct char_data *tch, *tch_next;
   int dir, found = 0;
   char *heard_nothing = "You don't hear anything unusual.\r\n";
   char *room_spiel    = "$n seems to listen intently for something.";
	char buf[MAX_INPUT_LENGTH];
	
	int roll = rand_number(1, 101);
	int goal = GET_SKILL(ch, SKILL_LISTEN);
	goal += dex_app_skill[GET_DEX(ch)].listen; /* Should be 0... */
	goal += race_app_skill[GET_RACE(ch)].listen;

   if (roll > goal) {
      send_to_char(ch, heard_nothing);
		WAIT_STATE(ch, PULSE_VIOLENCE);
      return;
   }

   one_argument(argument, buf);

   if (!*buf) {
      /* no argument means that the character is listening for
       * hidden or invisible beings in the room he/she is in
       */
      for (tch = world[IN_ROOM(ch)].people; tch; tch = tch_next) {
         tch_next = tch->next_in_room;
         if (tch != ch && !CAN_SEE(ch, tch) && GET_LEVEL(tch) < LVL_IMMORT)
            found++;
      }

      if (found) {
         if (GET_LEVEL(ch) >= 15) {
            /* being a higher level is better */
            send_to_char(ch, "You hear what might be %d hidden or invisible "
								 "creatures.\r\n",
								 MAX(1,(found+rand_number(0,1)-rand_number(0,1))));
         } else 
				send_to_char(ch, "You hear an odd rustling in the immediate area.\r\n");
      } else
			send_to_char(ch, heard_nothing);

      act(room_spiel, TRUE, ch, 0, 0, TO_ROOM);
		WAIT_STATE(ch, PULSE_VIOLENCE*2);
      return;
   } else {
      /* the argument must be one of the cardinal directions: north,
       * south, etc.
       */
      for (dir = 0; dir < NUM_OF_DIRS; dir++) {
         if (!strncmp(buf, dirs[dir], strlen(buf)))
            break;
      }

      if (dir == NUM_OF_DIRS) {
         send_to_char(ch, "Listen where?\r\n");
         return;
      }

      if (CAN_GO(ch, dir) || CAN_LISTEN_BEHIND_DOOR(ch, dir)) {
         for (tch = world[EXIT(ch, dir)->to_room].people; tch; tch=tch_next) {
            tch_next = tch->next_in_room;
            found++;
         }

         if (found) {
            if (GET_LEVEL(ch) >= 15) {
               send_to_char(ch, "You hear what might be %d creatures %s%s.\r\n",
								MAX(1,(found+rand_number(0,1)-rand_number(0,1))),
								(dir == 5 ? "below" : dir == 4 ? "above" : "to the "),
								(dir == 5 ? "" : dir == 4 ? "" : dirs[dir]));
            } else 
					send_to_char(ch, "You hear sounds from %s%s.\r\n", 
								(dir == 5 ? "below" : dir == 4 ? "above" : "the "),
								(dir == 5 ? "" : dir == 4 ? "" : dirs[dir]));
         } else 
				send_to_char(ch, heard_nothing);

         act(room_spiel, TRUE, ch, 0, 0, TO_ROOM);
			WAIT_STATE(ch, PULSE_VIOLENCE*2);
         return;
      } else 
			send_to_char(ch, "You can't listen in that direction.\r\n");
   }
	WAIT_STATE(ch, PULSE_VIOLENCE);
}


ACMD(do_farscan)
{
	struct char_data *i;
	int is_in, dir, dis, maxdis, found = 0;
	char buf[MAX_STRING_LENGTH];

	const char *distance[] = {
		"right here",
		"immediately ",
		"nearby ",
		"a ways ",
		"far ",
		"very far ",
		"extremely far ",
		"impossibly far ",
	};

	if (!GET_SKILL(ch, SKILL_FARSCAN)) {
		do_scan(ch, argument, cmd, subcmd);
		return;
	}

	if (AFF_FLAGGED(ch, AFF_BLIND)) {
		act("You can't see anything, you're blind!", TRUE, ch, 0, 0, TO_CHAR);
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	if (GET_MOVE(ch) < 5 && GET_LEVEL(ch) < LVL_IMMORT) {
		act("You are too exhausted.", TRUE, ch, 0, 0, TO_CHAR);
		return;
	}

	maxdis = (1 + ((GET_SKILL(ch, SKILL_FARSCAN) * 5) / 100));
	if (GET_LEVEL(ch) >= LVL_IMMORT)
		maxdis = 7;

	act("You quickly scan the area and see:", TRUE, ch, 0, 0, TO_CHAR);
	act("$n quickly scans the area.", FALSE, ch, 0, 0, TO_ROOM);
	if (GET_LEVEL(ch) < LVL_IMMORT)
		GET_MOVE(ch) -= 5;

	is_in = IN_ROOM(ch);
	for (dir = 0; dir < NUM_OF_DIRS; dir++) {
		IN_ROOM(ch) = is_in;

		for (dis = 0; dis <= maxdis; dis++) {
			if ((dis == 0 && dir == 0) || dis > 0) {
				for (i = world[IN_ROOM(ch)].people; i; i = i->next_in_room) {
					if ((!(ch == i && dis == 0)) && CAN_SEE(ch, i)) {
						snprintf(buf, sizeof(buf), "%33s: %s%s%s%s", 
								GET_NAME(i), distance[dis],
								(dis > 0 && dir < (NUM_OF_DIRS - 2)) ? "to the " : "",
								dis > 0 ? dirs[dir] : "",
								(dis > 0 && dir > (NUM_OF_DIRS - 3)) ? "wards" : "");
						act(buf, TRUE, ch, 0, 0, TO_CHAR);
						found++;
					}
				}
			}
			if (!CAN_GO(ch, dir) || 
				 (world[IN_ROOM(ch)].dir_option[dir]->to_room == is_in))
				break;
			else
				IN_ROOM(ch) = world[IN_ROOM(ch)].dir_option[dir]->to_room;
		}
	}
	if (found == 0)
		act("Nobody anywhere near you.", TRUE, ch, 0, 0, TO_CHAR);

	IN_ROOM(ch) = is_in;
}


ACMD(do_poison)
{
	struct obj_data *target, *source, *next_obj;
	struct char_data *vict;
	char arg[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	int roll, goal;

	if (!GET_SKILL(ch, SKILL_POISON)) {
		send_to_char(ch, "You don't know how!\r\n");
		return;
	}

	two_arguments(argument, arg, buf);

	if (!*arg) {
		send_to_char(ch, "Who's drink do you want to poison?\r\n");
		return;
	}

	if (!(vict = get_char_room_vis(ch, arg, NULL))) {
		send_to_char(ch, "Your victim does not seem to be in the room.\r\n");
		return;
	}
	if (!*buf) {
		send_to_char(ch, "Which item of %s's do you wish to poison?\r\n", 
						 GET_NAME(vict));
		return;
	}

	if (!(target = get_obj_in_list_vis(vict, buf, NULL, vict->carrying))) {
		send_to_char(ch, "%s doesn't have one of those.\r\n", GET_NAME(vict));
		return;
	}


	for (target = vict->carrying; target; target = next_obj) {
		next_obj = target->next_content;
		if (CAN_SEE_OBJ(ch, target) && GET_OBJ_TYPE(target) == ITEM_DRINKCON) {
			if (GET_OBJ_VAL(target, 1) <= 0) {
				send_to_char(ch, "Drat! %s's %s is currently empty. You'll wait "
								 "until it gets filled up.\r\n", 
								 GET_NAME(vict), target->short_description);
				return;
			}
			if (GET_OBJ_VAL(target, 3)) {
				send_to_char(ch, "The contents of %s are already poisoned.\r\n"
								 "Looks like someone else already beat you to it!\r\n",
								 target->short_description);
				return;
			}
			roll = rand_number(1, 101);
			goal = GET_SKILL(ch, SKILL_POISON);
			if (AFF_FLAGGED(ch, AFF_SNEAK))
				goal += 25;

			for (source = ch->carrying; source; source = next_obj) {
				next_obj = source->next_content;
				if (CAN_SEE_OBJ(ch, source) && 
					 GET_OBJ_TYPE(source) == ITEM_DRINKCON) {
					if (GET_OBJ_VAL(source, 3) <= 0) {
						send_to_char(ch, "How can you poison another drink with a "
										 "liquid that isn't itself poisoned?!\r\n");
						return;
					}
					if (GET_OBJ_VAL(source, 3) >= 1 && roll <= goal) {
						send_to_char(ch, "You unobtrusively slip the contents of "
										 "%s into %s's %s.\r\n",
										 source->short_description, GET_NAME(vict), 
										 target->short_description);
						GET_OBJ_VAL(target, 3)++;
						/* The cost of success is to use up all the source poison */
						GET_OBJ_VAL(source, 1) = 0;
						return;
					}
					send_to_char(ch, "Uh oh. Looks like you've been caught!\r\n");
					act("$n just tried to poison your $p!", 
						 FALSE, ch, target, 0, TO_VICT);
					act("$n just tried to poison $N's $p!", 
						 FALSE, ch, target, vict, TO_NOTVICT);
					WAIT_STATE(ch, 10);
					return;
				}
			}
			send_to_char(ch, "You have nothing in your inventory which could be "
							 "used to poison.\r\n");
			return;
		}
	}
}


ACMD(do_sap)
{
	struct affected_type af;
	struct char_data *vict;
	char buf[MAX_INPUT_LENGTH];
	int goal = GET_SKILL(ch, SKILL_SAP);
	int roll = rand_number(1, 101);

	one_argument(argument, buf);

	if (!(vict = get_char_room_vis(ch, buf, NULL))) {
		send_to_char(ch,"Sap who?\r\n");
		return;
	}

	if (vict == ch) {
		send_to_char(ch, "Try <sleep> instead.\r\n");
		return;
	}

	if (!GET_EQ(ch, WEAR_WIELD)) {
		send_to_char(ch, "You need to wield a weapon to make it a success.\r\n");
		return;
	}

	/* Bludgeoning weapons only, but what defines bludgeoning?  Adjust
	 * to your own taste.  -ejg */
	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_BLUDGEON - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_CRUSH - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_POUND - TYPE_HIT &&
		 GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != TYPE_MAUL - TYPE_HIT) {
		send_to_char(ch, "Only bludgeoning weapons can be used for "
						 "sapping.\r\n");
		return;
	}

	if (FIGHTING(vict)) {
		send_to_char(ch, "You can't sap a fighting person -- they're "
						 "too alert!\r\n");
		return;
	}

	if (!AFF_FLAGGED(ch, AFF_SNEAK) ||
		 ((MOB_FLAGGED(vict, MOB_AWARE) || AFF_FLAGGED(vict, AFF_AWARE)) && 
		  AWAKE(vict))) {
		act("You notice $N raising $s weapon at you!", 
			 FALSE, vict, 0, ch, TO_CHAR);
		act("$e notices you raising your weapon!",
			 FALSE, vict, 0, ch, TO_VICT);
		act("$n notices $N raising $s weapon at $m!",
			 FALSE, vict, 0, ch, TO_NOTVICT);

		hit(vict, ch, TYPE_UNDEFINED);
		return;
	}

	goal -= GET_LEVEL(vict);

	if (AWAKE(vict) && (roll <= goal)) {
		act("You are knocked out when $N hits you upside your head.", 
			 FALSE, vict, 0, ch, TO_CHAR);
		act("$n sees stars and slumps over, knocked out.", 
			 FALSE, vict, 0, ch, TO_VICT);
		act("$n sees stars and slumps over, knocked out, after $N brains $m.", 
			 FALSE, vict, 0, ch, TO_NOTVICT);
    
		af.type = SKILL_SAP;
		af.duration = GET_LEVEL(ch) / 5;
		af.modifier = 0;
		af.location = APPLY_NONE;

		EMPTY(af.bitvector)
		af.bitvector.AFF_SLEEP = 1;

		affect_join(vict, &af, FALSE, FALSE, FALSE, FALSE);
		if (GET_POS(vict) > POS_SLEEPING)
			GET_POS(vict) = POS_SLEEPING;

	}  else {
		act("You notice $N raising $s weapon at you!", 
			 FALSE, vict, 0, ch, TO_CHAR);
		act("$e notices you raising your weapon!", 
			 FALSE, vict, 0, ch, TO_VICT);
		act("$n notices $N raising $s weapon at $m!",
			 FALSE, vict, 0, ch, TO_NOTVICT);
		hit(ch, vict, SKILL_SAP);
	}
}
