/* to do_stat and support functions */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "screen.h"
#include "spells.h"


/* extern vars */
extern struct attack_hit_type attack_hit_text[NUM_ATTACK_TYPES];
extern const char *pc_class_types[];
extern const char *pc_race_types[];
extern int load_into_inventory;


/* local functions */
void do_stat_room(struct char_data *ch);
void do_stat_object(struct char_data *ch, struct obj_data *j);
void do_stat_character(struct char_data *ch, struct char_data *k);
void do_zstat_room(struct char_data *ch, zone_rnum rnum);
int find_attack_type(char *arg);

ACMD(do_apply);
ACMD(do_load);
ACMD(do_stat);
ACMD(do_vlist);
ACMD(do_vnum);
ACMD(do_vstat);


void do_stat_room(struct char_data *ch)
{
	char buf2[MAX_STRING_LENGTH];
	struct extra_descr_data *desc;
	struct room_data *rm = &world[IN_ROOM(ch)];
	int i, found, column;
	struct obj_data *j;
	struct char_data *k;

	send_to_char(ch, "Room name: %s%s%s\r\n", CCCYN(ch, C_NRM), rm->name, CCNRM(ch, C_NRM));

	sprinttype(rm->sector_type, sector_types, buf2, sizeof(buf2));
	send_to_char(ch, "Zone: [%3d], VNum: [%s%5d%s], RNum: [%5d], Type: %s\r\n",
					 zone_table[rm->zone].number, CCGRN(ch, C_NRM), rm->number,
					 CCNRM(ch, C_NRM), IN_ROOM(ch), buf2);

	sprintfield((char*)&rm->room_flags, room_bits, buf2, sizeof(buf2));
	send_to_char(ch, "SpecProc: %s, Flags: %s\r\n", rm->func == NULL ? "None" : "Exists", buf2);

	sprintfield((char*)&rm->room_affections, room_affections, buf2, sizeof(buf2));
	send_to_char(ch, "Room affections: %s\r\n", buf2);

	send_to_char(ch, "Description:\r\n%s", rm->description ? rm->description : "  None.\r\n");

	if (rm->ex_description) {
		send_to_char(ch, "Extra descs:%s", CCCYN(ch, C_NRM));
		for (desc = rm->ex_description; desc; desc = desc->next)
			send_to_char(ch, " %s", desc->keyword);
		send_to_char(ch, "%s\r\n", CCNRM(ch, C_NRM));
	}

	send_to_char(ch, "Chars present:%s", CCYEL(ch, C_NRM));
	column = 14;	/* ^^^ strlen ^^^ */
	for (found = FALSE, k = rm->people; k; k = k->next_in_room) {
		if (!CAN_SEE(ch, k))
			continue;

		column += send_to_char(ch, "%s %s(%s)", found++ ? "," : "", GET_NAME(k),
									  !IS_NPC(k) ? "PC" : (!IS_MOB(k) ? "NPC" : "MOB"));
		if (column >= 62) {
			send_to_char(ch, "%s\r\n", k->next_in_room ? "," : "");
			found = FALSE;
			column = 0;
		}
	}
	send_to_char(ch, "%s", CCNRM(ch, C_NRM));

	if (rm->contents) {
		send_to_char(ch, "Contents:%s", CCGRN(ch, C_NRM));
		column = 9;	/* ^^^ strlen ^^^ */

		for (found = 0, j = rm->contents; j; j = j->next_content) {
			if (!CAN_SEE_OBJ(ch, j))
				continue;

			column += send_to_char(ch, "%s %s", found++ ? "," : "", j->short_description);
			if (column >= 62) {
				send_to_char(ch, "%s\r\n", j->next_content ? "," : "");
				found = FALSE;
				column = 0;
			}
		}
		send_to_char(ch, "%s", CCNRM(ch, C_NRM));
	}

	for (i = 0; i < NUM_OF_DIRS; i++) {
		char buf1[128];

		if (!rm->dir_option[i])
			continue;

		if (rm->dir_option[i]->to_room == NOWHERE)
			snprintf(buf1, sizeof(buf1), " %sNONE%s", CCCYN(ch, C_NRM), CCNRM(ch, C_NRM));
		else
			snprintf(buf1, sizeof(buf1), "%s%5d%s", CCCYN(ch, C_NRM), GET_ROOM_VNUM(rm->dir_option[i]->to_room), CCNRM(ch, C_NRM));

      sprintfield((char*)&rm->dir_option[i]->exit_info, exit_bits, buf2, sizeof(buf2));

		send_to_char(ch, "Exit %s%-5s%s:  To: [%s], Key: [%5d], Keywrd: %s, Type: %s\r\n%s",
						 CCCYN(ch, C_NRM), dirs[i], CCNRM(ch, C_NRM), buf1, rm->dir_option[i]->key,
						 rm->dir_option[i]->keyword ? rm->dir_option[i]->keyword : "None", buf2,
						 rm->dir_option[i]->general_description ? rm->dir_option[i]->general_description : "  No exit description.\r\n");
	}
}



void do_stat_object(struct char_data *ch, struct obj_data *j)
{
	int i, found;
	obj_vnum vnum;
	struct obj_data *j2;
	struct extra_descr_data *desc;
	struct char_data *tempch;
	char buf[MAX_STRING_LENGTH];

	vnum = GET_OBJ_VNUM(j);
	send_to_char(ch, "Name: '%s%s%s', Aliases: %s\r\n", CCYEL(ch, C_NRM),
					 j->short_description ? j->short_description : "<None>",
					 CCNRM(ch, C_NRM), j->name);

	sprinttype(GET_OBJ_TYPE(j), item_types, buf, sizeof(buf));
	send_to_char(ch, "VNum: [%s%5d%s], RNum: [%5d], Type: %s, SpecProc: %s\r\n",
					 CCGRN(ch, C_NRM), vnum, CCNRM(ch, C_NRM), GET_OBJ_RNUM(j), buf,
					 GET_OBJ_SPEC(j) ? "Exists" : "None");

	if (j->ex_description) {
		send_to_char(ch, "Extra descs:%s", CCCYN(ch, C_NRM));
		for (desc = j->ex_description; desc; desc = desc->next)
			send_to_char(ch, " %s", desc->keyword);
		send_to_char(ch, "%s\r\n", CCNRM(ch, C_NRM));
	}

	sprintfield((char*)&j->obj_flags.wear_flags, wear_bits, buf, sizeof(buf));
	send_to_char(ch, "Can be worn on: %s\r\n", buf);

	sprintfield((char*)&j->obj_flags.bitvector, affected_bits, buf, sizeof(buf));
	send_to_char(ch, "Set char bits : %s\r\n", buf);

	sprintfield((char*)&GET_OBJ_EXTRA(j), extra_bits, buf, sizeof(buf));
	send_to_char(ch, "Extra flags   : %s\r\n", buf);

	send_to_char(ch, "Weight: %d, Value: %d, Cost/day: %d, Timer: %d\r\n",
					 GET_OBJ_WEIGHT(j), GET_OBJ_COST(j), GET_OBJ_RENT(j), GET_OBJ_TIMER(j));

	send_to_char(ch, "In room: %d (%s), ", GET_ROOM_VNUM(IN_ROOM(j)),
					 IN_ROOM(j) == NOWHERE ? "Nowhere" : world[IN_ROOM(j)].name);

	/*
	 * NOTE: In order to make it this far, we must already be able to see the
	 *       character holding the object. Therefore, we do not need CAN_SEE().
	 */
	send_to_char(ch, "In object: %s, ", IN_OBJ(j) ? IN_OBJ(j)->short_description : "None");
	send_to_char(ch, "Carried by: %s, ", CARRIED_BY(j) ? GET_NAME(CARRIED_BY(j)) : "Nobody");
	send_to_char(ch, "Worn by: %s\r\n", WORN_BY(j) ? GET_NAME(WORN_BY(j)) : "Nobody");

	switch (GET_OBJ_TYPE(j)) {
		case ITEM_LIGHT:
			if (GET_OBJ_VAL(j, 2) == -1)
				send_to_char(ch, "Hours left: Infinite\r\n");
			else
				send_to_char(ch, "Hours left: [%d]\r\n", GET_OBJ_VAL(j, 2));
			break;
		case ITEM_SCROLL:
		case ITEM_POTION:
			send_to_char(ch, "Spells: (Level %d) %s, %s, %s\r\n", GET_OBJ_VAL(j, 0),
							 skill_name(GET_OBJ_VAL(j, 1)), skill_name(GET_OBJ_VAL(j, 2)),
							 skill_name(GET_OBJ_VAL(j, 3)));
			break;
		case ITEM_WAND:
		case ITEM_STAFF:
			send_to_char(ch, "Spell: %s at level %d, %d (of %d) charges remaining\r\n",
							 skill_name(GET_OBJ_VAL(j, 3)), GET_OBJ_VAL(j, 0),
							 GET_OBJ_VAL(j, 2), GET_OBJ_VAL(j, 1));
			break;
		case ITEM_WEAPON:
			send_to_char(ch, "Todam: %dd%d, Message type: %d\r\n",
							 GET_OBJ_VAL(j, 1), GET_OBJ_VAL(j, 2), GET_OBJ_VAL(j, 3));
			break;
		case ITEM_ARMOR:
			send_to_char(ch, "AC-apply: [%d]\r\n", GET_OBJ_VAL(j, 0));
			break;
		case ITEM_TRAP:
			send_to_char(ch, "Spell: %d, - Hitpoints: %d\r\n", GET_OBJ_VAL(j, 0), GET_OBJ_VAL(j, 1));
			break;
		case ITEM_CONTAINER:
			sprintbit(GET_OBJ_VAL(j, 1), container_bits, buf, sizeof(buf));
			send_to_char(ch, "Weight capacity: %d, Lock Type: %s, Key Num: %d, Corpse: %s\r\n",
							 GET_OBJ_VAL(j, 0), buf, GET_OBJ_VAL(j, 2),
							 YESNO(GET_OBJ_VAL(j, 3)));
			break;
		case ITEM_DRINKCON:
		case ITEM_FOUNTAIN:
			sprinttype(GET_OBJ_VAL(j, 2), drinks, buf, sizeof(buf));
			send_to_char(ch, "Capacity: %d, Contains: %d, Poisoned: %s, Liquid: %s\r\n",
							 GET_OBJ_VAL(j, 0), GET_OBJ_VAL(j, 1), YESNO(GET_OBJ_VAL(j, 3)), buf);
			break;
		case ITEM_NOTE:
			send_to_char(ch, "Tongue: %d\r\n", GET_OBJ_VAL(j, 0));
			break;
		case ITEM_KEY:
			/* Nothing */
			break;
		case ITEM_FOOD:
			send_to_char(ch, "Makes full: %d, Poisoned: %s\r\n", GET_OBJ_VAL(j, 0), YESNO(GET_OBJ_VAL(j, 3)));
			break;
		case ITEM_MONEY:
			send_to_char(ch, "Copper: %d, Silver: %d, Gold: %d, Plat: %d\r\n", 
							 GET_OBJ_VAL(j, COPPER_COINS), 
							 GET_OBJ_VAL(j, SILVER_COINS), 
							 GET_OBJ_VAL(j, GOLD_COINS), 
							 GET_OBJ_VAL(j, PLAT_COINS));
			break;
		case ITEM_CHAIR:
			send_to_char(ch, "Can hold: [%d] Num. of People in Chair: [%d]\r\n",
							 GET_OBJ_VAL(j, 0), GET_OBJ_VAL(j, 1));
			send_to_char(ch, "Holding : ");
			for (tempch = OBJ_SAT_IN_BY(j); tempch; tempch = NEXT_SITTING(tempch)){
				send_to_char(ch, "%s ", GET_NAME(tempch));
			}
			break;
		case ITEM_BOW:
		case ITEM_CROSSBOW:
		case ITEM_SLING:
			send_to_char(ch, "Range: %d\r\n", GET_OBJ_VAL(j, 1));
			break;
		case ITEM_THROW:
		case ITEM_ROCK:
		case ITEM_BOLT:
		case ITEM_ARROW:
			send_to_char(ch, "Number dam dice: %d, Size dam dice: %d\r\n",
							 GET_OBJ_VAL(j, 2), GET_OBJ_VAL(j, 3));
			break;
		case ITEM_GRENADE:
			send_to_char(ch, "Timer: %d, Num dam dice: %d, Size dam dice: %d\r\n",
							 GET_OBJ_VAL(j, 0), GET_OBJ_VAL(j, 1), GET_OBJ_VAL(j, 2));
			break;
		default:
			send_to_char(ch, "Values 0-3: [%d] [%d] [%d] [%d]\r\n",
							 GET_OBJ_VAL(j, 0), GET_OBJ_VAL(j, 1),
							 GET_OBJ_VAL(j, 2), GET_OBJ_VAL(j, 3));
			break;
	}

	/*
	 * I deleted the "equipment status" code from here because it seemed
	 * more or less useless and just takes up valuable screen space.
	 */

	if (j->contains) {
		int column;

		send_to_char(ch, "\r\nContents:%s", CCGRN(ch, C_NRM));
		column = 9;	/* ^^^ strlen ^^^ */

		for (found = 0, j2 = j->contains; j2; j2 = j2->next_content) {
			column += send_to_char(ch, "%s %s", found++ ? "," : "", j2->short_description);
			if (column >= 62) {
				send_to_char(ch, "%s\r\n", j2->next_content ? "," : "");
				found = FALSE;
				column = 0;
			}
		}
		send_to_char(ch, "%s", CCNRM(ch, C_NRM));
	}

	found = FALSE;
	send_to_char(ch, "Affections:");
	for (i = 0; i < MAX_OBJ_AFFECT; i++)
		if (j->affected[i].modifier) {
			sprinttype(j->affected[i].location, apply_types, buf, sizeof(buf));
			send_to_char(ch, "%s %+d to %s", found++ ? "," : "", j->affected[i].modifier, buf);
		}
	if (!found)
		send_to_char(ch, " None");

	send_to_char(ch, "\r\n");
}


void do_stat_character(struct char_data *ch, struct char_data *k)
{
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	int i, i2, column, found = FALSE;
	struct obj_data *j;
	struct follow_type *fol;
	struct affected_type *aff;

	sprinttype(GET_SEX(k), genders, buf, sizeof(buf));
	send_to_char(ch, "%s %s '%s'  IDNum: [%5ld], In room [%5d]\r\n",
					 buf, (!IS_NPC(k) ? "PC" : (!IS_MOB(k) ? "NPC" : "MOB")),
					 GET_NAME(k), GET_IDNUM(k), GET_ROOM_VNUM(IN_ROOM(k)));

	if (IS_MOB(k))
		send_to_char(ch, "Alias: %s, VNum: [%5d], RNum: [%5d]\r\n", 
						 k->player.name, GET_MOB_VNUM(k), GET_MOB_RNUM(k));

	send_to_char(ch, "Title: %s\r\n", 
					 k->player.title ? k->player.title : "<None>");

	send_to_char(ch, "L-Des: %s", 
					 k->player.long_descr ? k->player.long_descr : "<None>\r\n");

	sprinttype(k->player.chclass, IS_NPC(k) ? npc_class_types : pc_class_types, buf, sizeof(buf));
	sprinttype(k->player.race, pc_race_types, buf2, sizeof(buf2));
	send_to_char(ch, 
					 "%sClass: %s, Race: %s, Lev: [%s%2d%s], XP: [%s%7d%s], "
					 "Align: [%4d]\r\n", IS_NPC(k) ? "Monster " : "", buf, buf2, 
					 CCYEL(ch, C_NRM), GET_LEVEL(k), CCNRM(ch, C_NRM),
					 CCYEL(ch, C_NRM), GET_EXP(k), CCNRM(ch, C_NRM), 
					 GET_ALIGNMENT(k));

	if (!IS_NPC(k)) {
		char buf1[64], buf2[64];

		strlcpy(buf1, asctime(localtime(&(k->player.time.birth))), sizeof(buf1));
		strlcpy(buf2, asctime(localtime(&(k->player.time.logon))), sizeof(buf2));
		buf1[10] = buf2[10] = '\0';

		send_to_char(ch, "Created: [%s], Last Logon: [%s], Played [%dh %dm], "
						 "Age [%d]\r\n", buf1, buf2, k->player.time.played / 3600,
						 ((k->player.time.played % 3600) / 60), age(k)->year);

		send_to_char(ch, "Hometown: [%d], Speaks: [%d/%d/%d], (STL[%d]/"
						 "per[%d]/NSTL[%d])\r\n", k->player.hometown, GET_TALK(k, 0),
						 GET_TALK(k, 1), GET_TALK(k, 2), GET_PRACTICES(k),
						 int_app[GET_INT(k)].learn, wis_app[GET_WIS(k)].bonus);
	}
	send_to_char(ch, "Str: [%s%d/%d%s]  Int: [%s%d%s]  Wis: [%s%d%s]  "
					 "Dex: [%s%d%s]  Con: [%s%d%s]  Cha: [%s%d%s]\r\n",
					 CCCYN(ch, C_NRM), GET_STR(k), GET_ADD(k), CCNRM(ch, C_NRM),
					 CCCYN(ch, C_NRM), GET_INT(k), CCNRM(ch, C_NRM),
					 CCCYN(ch, C_NRM), GET_WIS(k), CCNRM(ch, C_NRM),
					 CCCYN(ch, C_NRM), GET_DEX(k), CCNRM(ch, C_NRM),
					 CCCYN(ch, C_NRM), GET_CON(k), CCNRM(ch, C_NRM),
					 CCCYN(ch, C_NRM), GET_CHA(k), CCNRM(ch, C_NRM));

	send_to_char(ch, "Hit p.:[%s%d/%d+%d%s]  Mana p.:[%s%d/%d+%d%s]  "
					 "Move p.:[%s%d/%d+%d%s]\r\n",
					 CCGRN(ch, C_NRM), GET_HIT(k), GET_MAX_HIT(k), 
					 hit_gain(k), CCNRM(ch, C_NRM),
					 CCGRN(ch, C_NRM), GET_MANA(k), GET_MAX_MANA(k), 
					 mana_gain(k), CCNRM(ch, C_NRM),
					 CCGRN(ch, C_NRM), GET_MOVE(k), GET_MAX_MOVE(k), 
					 move_gain(k), CCNRM(ch, C_NRM));

	send_to_char(ch, 
					 "Coins: [%s%9d%s]PP [%s%9d%s]GP [%s%9d%s]SP [%s%9d%s]CP\r\n"
					 "Bank : [%s%d%s] (Total: %s%d%sCP), Shares: [%s%9d%s]\r\n",
					 QWHT, GET_PLATINUM(k), QNRM, QYEL, GET_GOLD(k), QNRM, 
					 QCYN, GET_SILVER(k), QNRM, QMAG, GET_COPPER(k), QNRM, 
					 QYEL, GET_BANK_GOLD(k), QNRM, 
					 QMAG, convert_all_to_copper(k) + GET_BANK_GOLD(k), QNRM,
					 QGRN, GET_SHARES(k), QNRM);

	send_to_char(ch, "Private Channel: [%d]\r\n",
					 (!IS_NPC(k) ? GET_PRIVATE(k) : -1));

	send_to_char(ch, "AC: [%d%+d/10], Hitroll: [%2d], Damroll: [%2d], "
					 "Saving throws: [%d/%d/%d/%d/%d]\r\n", GET_AC(k), 
					 dex_app[GET_DEX(k)].defensive, k->points.hitroll,
					 k->points.damroll, GET_SAVE(k, 0), GET_SAVE(k, 1), 
					 GET_SAVE(k, 2), GET_SAVE(k, 3), GET_SAVE(k, 4));

	sprinttype(GET_POS(k), position_types, buf, sizeof(buf));
	send_to_char(ch, "Pos: %s, Fighting: %s", 
					 buf, FIGHTING(k) ? GET_NAME(FIGHTING(k)) : "Nobody");

	if (RIDING(k))
		send_to_char(ch, ", Riding %s\r\n", GET_NAME(RIDING(k)));
	else if (RIDDEN_BY(k))
		send_to_char(ch, ", Ridden by %s\r\n", GET_NAME(RIDDEN_BY(k)));

	if (IS_NPC(k))
		send_to_char(ch, "Attack type: %s", 
						 attack_hit_text[(int)k->mob_specials.attack_type].singular);

	if (k->desc) {
		sprinttype(STATE(k->desc), connected_types, buf, sizeof(buf));
		send_to_char(ch, ", Connected: %s", buf);
	}

	if (IS_NPC(k)) {
		sprinttype(k->mob_specials.default_pos, position_types, buf, sizeof(buf));
		send_to_char(ch, ", Default position: %s\r\n", buf);
		sprintfield((char*)&MOB_FLAGS(k), action_bits, buf, sizeof(buf));
		send_to_char(ch, "NPC flags: %s%s%s\r\n", 
						 CCCYN(ch, C_NRM), buf, CCNRM(ch, C_NRM));
	} else {
		send_to_char(ch, ", Idle Timer (in tics) [%d]\r\n", 
						 k->char_specials.timer);

		sprintfield((char*)&PLR_FLAGS(k), player_bits, buf, sizeof(buf));
		send_to_char(ch, "PLR: %s%s%s\r\n", 
						 CCCYN(ch, C_NRM), buf, CCNRM(ch, C_NRM));

		sprintfield((char*)&PRF_FLAGS(k), preference_bits, buf, sizeof(buf));
		send_to_char(ch, "PRF: %s%s%s\r\n", 
						 CCGRN(ch, C_NRM), buf, CCNRM(ch, C_NRM));
	}

	if (IS_MOB(k))
		send_to_char(ch, "Mob Spec-Proc: %s, NPC Bare Hand Dam: %dd%d\r\n",
						 (mob_index[GET_MOB_RNUM(k)].func ? "Exists" : "None"),
						 k->mob_specials.damnodice, k->mob_specials.damsizedice);

	for (i = 0, j = k->carrying; j; j = j->next_content, i++);
	send_to_char(ch, "Carried: weight: %d, items: %d; Items in: inventory: %d, ",
					 IS_CARRYING_W(k), IS_CARRYING_N(k), i);

	for (i = 0, i2 = 0; i < NUM_WEARS; i++)
		if (GET_EQ(k, i))
			i2++;
	send_to_char(ch, "eq: %d\r\n", i2);

	if (!IS_NPC(k))
		send_to_char(ch, "Hunger: %d, Thirst: %d, Drunk: %d, Sacrifices: %d\r\n",
						 GET_COND(k, FULL), GET_COND(k, THIRST), GET_COND(k, DRUNK),
						 NUM_SACS(k));

	column = send_to_char(ch, "Master is: %s, Followers are:", 
								 k->master ? GET_NAME(k->master) : "<none>");
	if (!k->followers)
		send_to_char(ch, " <none>\r\n");
	else {
		for (fol = k->followers; fol; fol = fol->next) {
			column += send_to_char(ch, "%s %s", found++ ? "," : "", 
										  PERS(fol->follower, ch));
			if (column >= 62) {
				send_to_char(ch, "%s\r\n", fol->next ? "," : "");
				found = FALSE;
				column = 0;
			}
		}
		if (column != 0)
			send_to_char(ch, "\r\n");
	}

	if (!IS_NPC(k))
		send_to_char(ch, "Contact: %s\r\n", 
						 (GET_CONTACT(k) ? GET_CONTACT(k) : "Nobody"));

	/* Showing the bitvector */
	sprintfield((char*)&AFF_FLAGS(k), affected_bits, buf, sizeof(buf));
	send_to_char(ch, "AFF: %s%s%s\r\n", 
					 CCYEL(ch, C_NRM), buf, CCNRM(ch, C_NRM));

	/* Routine to show what spells a char is affected by */
	if (k->affected) {
		for (aff = k->affected; aff; aff = aff->next) {
			send_to_char(ch, "SPL: (%3dhr) %s%-21s%s ", 
							 aff->duration + 1, CCCYN(ch, C_NRM), 
							 skill_name(aff->type), CCNRM(ch, C_NRM));

			if (aff->modifier)
				send_to_char(ch, "%+d to %s", 
								 aff->modifier, apply_types[(int) aff->location]);

			if (!ISEMPTY(aff->bitvector)) {
				if (aff->modifier)
					send_to_char(ch, ", ");

				sprintfield((char*)&aff->bitvector, affected_bits, buf, sizeof(buf));
				send_to_char(ch, "sets %s", buf);
			}
			send_to_char(ch, "\r\n");
		}
	}
}


ACMD(do_stat)
{
	char buf1[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];
	struct char_data *victim;
	struct obj_data *object;

	half_chop(argument, buf1, buf2);

	if (!*buf1) {
		send_to_char(ch, "Stats on who or what?\r\n");
		return;
	} else if (is_abbrev(buf1, "room")) {
		do_stat_room(ch);
	} else if (is_abbrev(buf1, "mob")) {
		if (!*buf2)
			send_to_char(ch, "Stats on which mobile?\r\n");
		else {
			if ((victim = get_char_vis(ch, buf2, NULL, FIND_CHAR_WORLD)) != NULL)
				do_stat_character(ch, victim);
			else
				send_to_char(ch, "No such mobile around.\r\n");
		}
	} else if (is_abbrev(buf1, "player")) {
		if (!*buf2) {
			send_to_char(ch, "Stats on which player?\r\n");
		} else {
			if ((victim = get_player_vis(ch, buf2, NULL, FIND_CHAR_WORLD)) != NULL)
				do_stat_character(ch, victim);
			else
				send_to_char(ch, "No such player around.\r\n");
		}
	} else if (is_abbrev(buf1, "file")) {
		if (!*buf2)
			send_to_char(ch, "Stats on which player?\r\n");
		else if ((victim = get_player_vis(ch, buf2, NULL, FIND_CHAR_WORLD)) != NULL)
			do_stat_character(ch, victim);
		else {
			CREATE(victim, struct char_data, 1);
			clear_char(victim);
			CREATE(victim->player_specials, struct player_special_data, 1);
			if (load_char(buf2, victim) >= 0) {
				char_to_room(victim, 0);
				if (GET_LEVEL(victim) > GET_LEVEL(ch))
					send_to_char(ch, "Sorry, you can't do that.\r\n");
				else
					do_stat_character(ch, victim);
				extract_char_final(victim);
			} else {
				send_to_char(ch, "There is no such player.\r\n");
				free_char(victim);
			}
		}
	} else if (is_abbrev(buf1, "object")) {
		if (!*buf2)
			send_to_char(ch, "Stats on which object?\r\n");
		else {
			if ((object = get_obj_vis(ch, buf2, NULL)) != NULL)
				do_stat_object(ch, object);
			else
				send_to_char(ch, "No such object around.\r\n");
		}
	} else {
		char *name = buf1;
		int number = get_number(&name);

		if ((object = get_obj_in_equip_vis(ch, name, &number, ch->equipment)) != NULL)
			do_stat_object(ch, object);
		else if ((object = get_obj_in_list_vis(ch, name, &number, ch->carrying)) != NULL)
			do_stat_object(ch, object);
		else if ((victim = get_char_vis(ch, name, &number, FIND_CHAR_ROOM)) != NULL)
			do_stat_character(ch, victim);
		else if ((object = get_obj_in_list_vis(ch, name, &number, world[IN_ROOM(ch)].contents)) != NULL)
			do_stat_object(ch, object);
		else if ((victim = get_char_vis(ch, name, &number, FIND_CHAR_WORLD)) != NULL)
			do_stat_character(ch, victim);
		else if ((object = get_obj_vis(ch, name, &number)) != NULL)
			do_stat_object(ch, object);
		else
			send_to_char(ch, "Nothing around by that name.\r\n");
	}
}


ACMD(do_vstat)
{
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];

	two_arguments(argument, buf, buf2);

	if (!*buf || !*buf2 || !isdigit(*buf2)) {
		send_to_char(ch, "Usage: vstat { obj | mob | room } <number>\r\n");
		return;
	}
	if (!is_number(buf2)) {
		send_to_char(ch, "That's not a valid number.\r\n");
		return;
	}

	if (is_abbrev(buf, "mob")) {
		struct char_data *mob;
		mob_rnum r_num;

		if ((r_num = real_mobile(atoi(buf2))) == NOBODY) {
			send_to_char(ch, "There is no monster with that number.\r\n");
			return;
		}
		mob = read_mobile(r_num, REAL);
		char_to_room(mob, 0);
		do_stat_character(ch, mob);
		extract_char(mob);
	} else if (is_abbrev(buf, "obj")) {
		struct obj_data *obj;
		obj_rnum r_num;

		if ((r_num = real_object(atoi(buf2))) == NOTHING) {
			send_to_char(ch, "There is no object with that number.\r\n");
			return;
		}
		obj = read_object(r_num, REAL);
		do_stat_object(ch, obj);
		extract_obj(obj);
	} else if (is_abbrev(buf, "room")){
		room_rnum r_num;

		if ((r_num = real_room(atoi(buf2))) == NOWHERE) {
			send_to_char(ch, "That room does not exist.\r\n");
			return;
		}
		do_zstat_room(ch, r_num);
	} else
		send_to_char(ch, "That'll have to be either 'obj' or 'mob' or 'room'.\r\n");
}


int find_attack_type(char *arg)
{
	int nr;

	for (nr = 0; nr < NUM_ATTACK_TYPES; nr++) {
		if (is_abbrev(arg, attack_hit_text[nr].singular))
			break;
	}

	return (nr);
}

ACMD(do_vnum)
{
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];

	half_chop(argument, buf, buf2);

	if (!*buf || !*buf2 || 
		 (!is_abbrev(buf, "mob") &&
		  !is_abbrev(buf, "obj") && 
		  !is_abbrev(buf, "room") && 
		  !is_abbrev(buf, "weapon"))) {
		send_to_char(ch, "Usage: vnum { obj | mob | room | weapon } "
						 "<name | attack-type>\r\n");
		return;
	}
	if (is_abbrev(buf, "mob"))
		if (!vnum_mobile(buf2, ch))
			send_to_char(ch, "No mobiles by that name.\r\n");

	if (is_abbrev(buf, "obj"))
		if (!vnum_object(buf2, ch))
			send_to_char(ch, "No objects by that name.\r\n");

	if (is_abbrev(buf, "weapon"))
		if (!vnum_weapon(find_attack_type(buf2), ch))
			send_to_char(ch, "No weapons with that attack-type.\r\n");

	if (is_abbrev(buf, "room"))
		if (!vnum_room(buf2, ch))
			send_to_char(ch, "No room name containing that word.\r\n");
}


ACMD(do_vlist)
{
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	int found = 0, choice = 0, nr = 0;
	zone_rnum zone;
	zone_vnum j;
  
	two_arguments(argument, buf, buf2);

  	if (is_abbrev(buf, "objects"))
		choice = 1;

	if (is_abbrev(buf, "mobiles"))
		choice = 2;

	if (is_abbrev(buf, "rooms"))
		choice = 3;

	if (!*buf || !*buf2 || !is_number(buf2) || !choice) {
		send_to_char(ch, "Usage: vlist { obj | mob } <zone number>\r\n");
		return;
	}
	j = atoi(buf2);
    
	for (zone = 0; zone <= top_of_zone_table; zone++) {
      if (zone_table[zone].number == j) {
			j = zone;
			break;
		}
	}
	
	if (j > top_of_zone_table) {
		send_to_char(ch, "That is not a vlid zone number..");
		return;
	}

	sprintf(buf, "Zone: %s\r\n", zone_table[j].name);
	
	switch (choice) {
		case 1:
		{
			for (nr = 0; nr <= top_of_objt; nr++) {
				if (obj_index[nr].vnum <= zone_table[j].top && 
					 obj_index[nr].vnum >= zone_table[j].bot) {
					sprintf(buf+strlen(buf), "%3d. [%5d] %s\r\n", ++found,
							  obj_index[nr].vnum, obj_proto[nr].short_description);
				}
			}
			
			break;
		}

		case 2:
		{
			for (nr = 0; nr <= top_of_mobt; nr++) {
				if (mob_index[nr].vnum <= zone_table[j].top &&
					 mob_index[nr].vnum >= zone_table[j].bot) {
					sprintf(buf+strlen(buf), "%3d. [%5d] %s\r\n", ++found,
							  mob_index[nr].vnum, GET_NAME(&mob_proto[nr]));
				}
			}
			
			break;
		}

		case 3:
		{
			for (nr = 0; nr <= top_of_world; nr++) {
				if (world[nr].zone == j) {
					sprintf(buf+strlen(buf), "%3d. [%5d] %s\r\n", ++found,
							  world[nr].number, world[nr].name);
				}
			}
			
			break;
		}

		default:
		{
			send_to_char(ch, "Come again?! :P");
			return;
		}
	}

	if (ch->desc)
		page_string(ch->desc, buf, TRUE);
}


ACMD(do_load)
{
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];

	two_arguments(argument, buf, buf2);

	if (!*buf || !*buf2 || !isdigit(*buf2)) {
		send_to_char(ch, "Usage: load { obj | mob } <number>\r\n");
		return;
	}
	if (!is_number(buf2)) {
		send_to_char(ch, "That is not a number.\r\n");
		return;
	}

	if (is_abbrev(buf, "mob")) {
		struct char_data *mob;
		mob_rnum r_num;

		if ((r_num = real_mobile(atoi(buf2))) == NOBODY) {
			send_to_char(ch, "There is no monster with that number.\r\n");
			return;
		}
		mob = read_mobile(r_num, REAL);
		char_to_room(mob, IN_ROOM(ch));

		act("$n makes a quaint, magical gesture with one hand.", TRUE, ch,
			 0, 0, TO_ROOM);
		act("$n has created $N!", FALSE, ch, 0, mob, TO_ROOM);
		act("You create $N.", FALSE, ch, 0, mob, TO_CHAR);
	} else if (is_abbrev(buf, "obj")) {
		struct obj_data *obj;
		obj_rnum r_num;

		if ((r_num = real_object(atoi(buf2))) == NOTHING) {
			send_to_char(ch, "There is no object with that number.\r\n");
			return;
		}
		obj = read_object(r_num, REAL);
		if (load_into_inventory)
			obj_to_char(obj, ch);
		else
			obj_to_room(obj, IN_ROOM(ch));
		act("$n makes a strange magical gesture.", TRUE, ch, 0, 0, TO_ROOM);
		act("$n has created $p!", FALSE, ch, obj, 0, TO_ROOM);
		act("You create $p.", FALSE, ch, obj, 0, TO_CHAR);
	} else
		send_to_char(ch, "That'll have to be either 'obj' or 'mob'.\r\n");
}


/*
  From: Frollo <Mudaholic@aol.com>
  Subject: Add an object apply
*/
ACMD(do_apply)
{
	int nr, found = FALSE, i, x, y = 0, plus = FALSE, minus = FALSE;
	char arg[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];
	char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];

	two_arguments(argument, arg, arg2);

	if (!strcmp(arg2, "+"))
		plus = TRUE;

	if (!strcmp(arg2, "-"))
		minus = TRUE;

	for (x = 1; x < 28; x++) {
		if (is_abbrev(arg, apply_types[x])) {
			y = x;
			found = TRUE;
			x = 27;
		}
	}

	if (!found || !*arg) {
		sprintf(buf, "Usage: apply <stat> (+/-)\r\n   ");

		for (x = 1; x < NUM_APPLY; x++) {
			sprintf(buf+strlen(buf), "%s%s%s", 
					  apply_types[x], x < 27 ? ", " : "", 
					  (x == 11 || x == 20 || x == 25 || x == 27) ? "\r\n   " : "");
		}
		strcat(buf, "\r\n");
      if (ch->desc)
			page_string(ch->desc, buf, TRUE);

		return;
	}

	found = FALSE;
	sprintf(buf, "Apply to %s:\r\n", apply_types[y]);
	send_to_char(ch, buf);

	for (nr = 0; nr <= top_of_objt; nr++) {
		for (i = 0; i < MAX_OBJ_AFFECT; i++) {
			if ((minus && obj_proto[nr].affected[i].modifier < 0) || 
				 (plus && obj_proto[nr].affected[i].modifier > 0) || 
				 (!minus && !plus && obj_proto[nr].affected[i].modifier)) {
				if (obj_proto[nr].affected[i].location == y) {
					sprinttype(obj_proto[nr].affected[i].location, apply_types,
								  buf2, sizeof(buf2));
					sprintf(buf + strlen(buf), "[%5d] %-45s %+d to %s\r\n", 
							  obj_index[nr].vnum, obj_proto[nr].short_description,
							  obj_proto[nr].affected[i].modifier, buf2);
					found = TRUE;
				}
			}
		}
	}

	if (!found)
		send_to_char(ch, " None.\r\n");
	else {
      if (ch->desc)
			page_string(ch->desc, buf, TRUE);
	}
}


ACMD(do_vwear)
{
	char buf[MAX_INPUT_LENGTH];
	struct can_be_worn_on wearpos;
	
	EMPTY(wearpos);
	one_argument(argument, buf);

	if (!*buf) {
		send_to_char(ch, 
						 "Usage: vwear <wear position | type>\r\n"
						 "Wear positions are:\r\n"
						 "finger  neck   body   head   legs    feet    hands    \r\n"
						 "shield  arms   about  waist  wrist   wield   hold     \r\n"
						 "------------------------------------------------------\r\n"
						 "Item types are:\r\n"
						 "light     scroll  wand   staff    treasure   armour   \r\n"
						 "potion    worn    other  trash    container  liquid   \r\n"
						 "key       food    money  pen      boat       fountain \r\n"
						 "pole      chair   throw  grenade  bow        sling    \r\n"
						 "crossbow  bolt    arrow  rock                         \r\n");
		return;
	}
	if (is_abbrev(buf, "finger")) {
		wearpos.ITEM_WEAR_FINGER = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "neck")) {
		wearpos.ITEM_WEAR_NECK = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "body")) {
		wearpos.ITEM_WEAR_BODY = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "head")) {
		wearpos.ITEM_WEAR_HEAD = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "legs")) {
		wearpos.ITEM_WEAR_LEGS = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "feet")) {
		wearpos.ITEM_WEAR_FEET = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "hands")) {
		wearpos.ITEM_WEAR_HANDS = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "arms")) {
		wearpos.ITEM_WEAR_ARMS = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "shield")) {
		wearpos.ITEM_WEAR_SHIELD = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "about body")) {
		wearpos.ITEM_WEAR_ABOUT = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "waist")) {
		wearpos.ITEM_WEAR_WAIST = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "wrist")) {
		wearpos.ITEM_WEAR_WRIST = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "wield")) {
		wearpos.ITEM_WEAR_WIELD = 1;
		vwear_object(wearpos, ch);
	} else if (is_abbrev(buf, "hold")) {
		wearpos.ITEM_WEAR_HOLD = 1;
		vwear_object(wearpos, ch);
	}
	
	else if (is_abbrev(buf, "light")) /* Start of new vwear- Item types */
		vwear_obj(ITEM_LIGHT, ch);
	else if (is_abbrev(buf, "scroll"))
		vwear_obj(ITEM_SCROLL, ch);
	else if (is_abbrev(buf, "wand"))
		vwear_obj(ITEM_WAND, ch);
	else if (is_abbrev(buf, "staff"))
		vwear_obj(ITEM_STAFF, ch);
	else if (is_abbrev(buf, "treasure"))
		vwear_obj(ITEM_TREASURE, ch);
	else if (is_abbrev(buf, "armor"))
		vwear_obj(ITEM_ARMOR, ch);
	else if (is_abbrev(buf, "potion"))
		vwear_obj(ITEM_POTION, ch);
	else if (is_abbrev(buf, "worn"))
		vwear_obj(ITEM_WORN, ch);
	else if (is_abbrev(buf, "other"))
		vwear_obj(ITEM_OTHER, ch);
	else if (is_abbrev(buf, "trash"))
		vwear_obj(ITEM_TRASH, ch);
	else if (is_abbrev(buf, "container"))
		vwear_obj(ITEM_CONTAINER, ch);
	else if (is_abbrev(buf, "liquid"))
		vwear_obj(ITEM_DRINKCON, ch);
	else if (is_abbrev(buf, "key"))
		vwear_obj(ITEM_KEY, ch);
	else if (is_abbrev(buf, "food"))
		vwear_obj(ITEM_FOOD, ch);
	else if (is_abbrev(buf, "money"))
		vwear_obj(ITEM_MONEY, ch);
	else if (is_abbrev(buf, "pen"))
		vwear_obj(ITEM_PEN, ch);
	else if (is_abbrev(buf, "boat"))
		vwear_obj(ITEM_BOAT, ch);
	else if (is_abbrev(buf, "fountain"))
		vwear_obj(ITEM_FOUNTAIN, ch);
	else if (is_abbrev(buf, "pole"))
		vwear_obj(ITEM_POLE, ch);
	else if (is_abbrev(buf, "chair"))
		vwear_obj(ITEM_CHAIR, ch);
	else if (is_abbrev(buf, "throw"))
		vwear_obj(ITEM_THROW, ch);
	else if (is_abbrev(buf, "grenade"))
		vwear_obj(ITEM_GRENADE, ch);
	else if (is_abbrev(buf, "bow"))
		vwear_obj(ITEM_BOW, ch);
	else if (is_abbrev(buf, "sling"))
		vwear_obj(ITEM_SLING, ch);
	else if (is_abbrev(buf, "crossbow"))
		vwear_obj(ITEM_CROSSBOW, ch);
	else if (is_abbrev(buf, "bolt"))
		vwear_obj(ITEM_BOLT, ch);
	else if (is_abbrev(buf, "arrow"))
		vwear_obj(ITEM_ARROW, ch);
	else if (is_abbrev(buf, "rock"))
		vwear_obj(ITEM_ROCK, ch);
	else
		send_to_char(ch, 
						 "Possible positions are:\r\n"
						 "Wear positions are:\r\n"
						 "finger  neck   body   head   legs    feet    hands    \r\n"
						 "shield  arms   about  waist  wrist   wield   hold     \r\n"
						 "------------------------------------------------------\r\n"
						 "Item types are:\r\n"
						 "light     scroll  wand   staff    treasure   armour   \r\n"
						 "potion    worn    other  trash    container  liquid   \r\n"
						 "key       food    money  pen      boat       fountain \r\n"
						 "pole      chair   throw  grenade  bow        sling    \r\n"
						 "crossbow  bolt    arrow  rock                         \r\n");
}
