/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/
 
/***************************************************************************
 *  The MOBprograms have been contributed by N'Atas-ha.  Any support for   *
 *  these routines should not be expected from Merc Industries.  However,  *
 *  under no circumstances should the blame for bugs, etc be placed on     *
 *  Merc Industries.  They are not guaranteed to work on all systems due   *
 *  to their frequent use of strxxx functions.  They are also not the most *
 *  efficient way to perform their tasks, but hopefully should be in the   *
 *  easiest possible way to install and begin using. Documentation for     *
 *  such installation can be found in INSTALL.  Enjoy........    N'Atas-Ha *
 ***************************************************************************/
 
#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "mobprog.h"
#include "screen.h"
 
 
/* extern functions */
room_rnum find_target_room(struct char_data *ch, char *rawroomstr);
ACMD(do_trans);

 
/* This routine transfers between alpha and numeric forms of the
 *  mob_prog bitvector types. It allows the words to show up in mpstat to
 *  make it just a hair bit easier to see what a mob should be doing.
 */
 
char *mprog_type_to_name(int type)
{
	switch (type)
	{
		case IN_FILE_PROG:          return ("in_file_prog");
		case ACT_PROG:              return ("act_prog");
		case SPEECH_PROG:           return ("speech_prog");
		case RAND_PROG:             return ("rand_prog");
		case FIGHT_PROG:            return ("fight_prog");
		case HITPRCNT_PROG:         return ("hitprcnt_prog");
		case DEATH_PROG:            return ("death_prog");
		case ENTRY_PROG:            return ("entry_prog");
		case GREET_PROG:            return ("greet_prog");
		case ALL_GREET_PROG:        return ("all_greet_prog");
		case GIVE_PROG:             return ("give_prog");
		case BRIBE_PROG:            return ("bribe_prog");
		default:                    return ("ERROR_PROG");
	}
}
 

/* string prefix routine */
int str_prefix(const char *astr, const char *bstr)
{
   if (!astr) {
		log("Strn_cmp: null astr.");
		return (TRUE);
   }
   if (!bstr) {
		log("Strn_cmp: null astr.");
		return (TRUE);
   }
   for (; *astr; astr++, bstr++) {
		if (LOWER(*astr) != LOWER(*bstr)) 
			return (TRUE);
   }
   return (FALSE);
}
 

/* A trivial rehack of do_mstat.  This doesnt show all the data, but just
 * enough to identify the mob and give its basic condition.  It does however,
 * show the MOBprograms which are set.
 */
void do_mpstat(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH ];
	MPROG_DATA *mprg;
	struct char_data  *victim;
 
	one_argument(argument, arg);
 
	if (arg[0] == '\0') {
		send_to_char(ch, "MobProg stat whom?\r\n");
		return;
	}
 
	if ((victim = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD)) == NULL) {
		send_to_char(ch, "They aren't here.\r\n");
		return;
	}
 
	if (!IS_NPC(victim)) {
		send_to_char(ch, "Only Mobiles can have Programs!\r\n");
		return;
	}
 
	if (!(mob_index[victim->nr].progtypes)) {
		send_to_char(ch, "That Mobile has no Programs set.\r\n");
		return;
	}
 
	send_to_char(ch, "Name: %s.  Vnum: %d.\r\n",
					 GET_NAME(victim), mob_index[victim->nr].vnum);
 
	send_to_char(ch, "Short description: %s.\r\nLong  description: %s",
					 victim->player.short_descr,
					 victim->player.long_descr[0] != '\0' ?
					 victim->player.long_descr : "(none).\r\n");
 
	send_to_char(ch, "Hp: %d/%d.  Mana: %d/%d.  Move: %d/%d. \r\n",
					 GET_HIT(victim), GET_MAX_HIT(victim),
					 GET_MANA(victim), GET_MAX_MANA(victim),
					 GET_MOVE(victim), GET_MAX_MOVE(victim));
 
	send_to_char(ch,
		"Lv: %d.  Class: %d, Align: %d, AC: %d, Wealth: %s%d%sCP, Exp: %d.\r\n",
					 GET_LEVEL(victim), GET_CLASS(victim),
					 GET_ALIGNMENT(victim), GET_AC(victim),
					 QMAG, convert_all_to_copper(victim),  QNRM, GET_EXP(victim));
 
	for (mprg = mob_index[victim->nr].mobprogs; mprg != NULL; mprg = mprg->next) {
		send_to_char(ch, ">%s %s\r\n%s\r\n", mprog_type_to_name(mprg->type),
						 mprg->arglist, mprg->comlist);
	}
}
 

/* prints the argument to all the rooms aroud the mobile */
void do_mpasound(struct char_data *ch, char *argument)
{
   room_vnum was_in_room;
   int  door;
   char arg[MAX_INPUT_LENGTH];
	struct room_direction_data *pexit;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	if (argument[0] == '\0') {
		log("Mpasound - No argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
	one_argument(argument, arg);
 
	was_in_room = IN_ROOM(ch);
	for (door = 0; door <= 5; door++) {
		if ((pexit = world[was_in_room].dir_option[door]) != NULL &&
			 pexit->to_room != NOWHERE && pexit->to_room != was_in_room) {
			IN_ROOM(ch) = pexit->to_room;
			MOBTrigger  = FALSE;
			act(arg, FALSE, ch, NULL, NULL, TO_ROOM);
		}
	}
 
   IN_ROOM(ch) = was_in_room;
}

 
/* lets the mobile kill any player or mobile without murder*/
void do_mpkill(struct char_data *ch, char *argument)
{
	char      arg[MAX_INPUT_LENGTH];
	struct char_data *victim;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	one_argument(argument, arg);
 
	if (arg[0] == '\0') {
		log("MpKill - no argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if ((victim = get_char_room_vis(ch, arg, NULL)) == NULL) {
		log("MpKill - Victim not in room: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (victim == ch) {
		log("MpKill - Bad victim to attack: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (AFF_FLAGGED(ch, AFF_CHARM) && ch->master == victim) {
		log("MpKill - Charmed mob attacking master: vnum %d.",
			  mob_index[ch->nr].vnum);
		return;
	}
 
	if (GET_POS(ch) == POS_FIGHTING) {	
		log("MpKill - Already fighting: vnum %d", mob_index[ch->nr].vnum);
		return;
	}
 
	hit(ch, victim, -1);
}
 
 
/* Let the mobile destroy an object in its inventory.  It can also
 *	destroy a worn object and it can destroy items using all.xxxxx or 
 *	just plain all of them. */
void do_mpjunk(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
	int pos;
	struct obj_data *obj;
	struct obj_data *obj_next;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	one_argument(argument, arg);
 
	if (arg[0] == '\0') {
		log("Mpjunk - No argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (str_cmp(arg, "all") && str_prefix("all.", arg)) {
		if ((obj = get_obj_in_equip_vis(ch, arg, NULL, ch->equipment))) {
			pos = get_obj_pos_in_equip_vis(ch, arg, NULL, ch->equipment);
			unequip_char(ch, pos);
			extract_obj(obj);
			return;
		}
		if ((obj = get_obj_in_list_vis(ch, arg, NULL, ch->carrying)) != NULL)
         extract_obj(obj);
		return;
	} else {
		for (obj = ch->carrying; obj != NULL; obj = obj_next) {
         obj_next = obj->next_content;
         if (arg[3] == '\0' || isname(arg+4, obj->name)) {
				extract_obj(obj);
         }
		}
		while((obj = get_obj_in_equip_vis(ch, arg, NULL, ch->equipment))) {
			pos = get_obj_pos_in_equip_vis(ch, arg, NULL, ch->equipment);
			unequip_char(ch, pos);
			extract_obj(obj);
		}   
	}
}

 
/* prints the message to everyone in the room other than the mob and victim */
void do_mpechoaround(struct char_data *ch, char *argument)
{
   char arg[MAX_INPUT_LENGTH];
   struct char_data *victim;
   char *p;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	p = one_argument(argument, arg);
	while(isspace(*p)) 
		p++; /* skip over leading space */
 
	if (arg[0] == '\0') {
		log("Mpechoaround - No argument:  vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (!(victim = get_char_room_vis(ch, arg, NULL))) {
		log("Mpechoaround - victim does not exist: vnum %d.",
			  mob_index[ch->nr].vnum);
		return;
	}
 
	act(p, FALSE, ch, NULL, victim, TO_NOTVICT);
}

 
/* prints the message to only the victim */
void do_mpechoat(struct char_data *ch, char *argument)
{
   char arg[MAX_INPUT_LENGTH];
   struct char_data *victim;
   char *p;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	p = one_argument(argument, arg);
	while(isspace(*p)) 
		p++; /* skip over leading space */
 
	if (arg[0] == '\0') {
		log("Mpechoat - No argument:  vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (!(victim = get_char_room_vis(ch, arg, NULL))) {
		log("Mpechoat - victim does not exist: vnum %d.",
			  mob_index[ch->nr].vnum);
		return;
	}
 
	act(p,FALSE,  ch, NULL, victim, TO_VICT);
}
 

/* prints the message to the room at large */
void do_mpecho(struct char_data *ch, char *argument)
{
	char *p;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	if (argument[0] == '\0') {
		log("Mpecho - called w/o argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
	p = argument;
	while(isspace(*p)) 
		p++;
 
	act(p,FALSE,  ch, NULL, NULL, TO_ROOM);
}

 
/* Let the mobile load an item or mobile.  All items are loaded
 * inventory.  You can specify a level with the load object portion as
 * well. */
void do_mpmload(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
	struct index_data *pMobIndex;
	struct char_data      *victim;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	one_argument(argument, arg);
 
	if (arg[0] == '\0' || !is_number(arg)) {
		log("Mpmload - Bad vnum as arg: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if ((pMobIndex = get_mob_index(atoi(arg))) == NULL) {
		log("Mpmload - Bad mob vnum: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	victim = read_mobile(atoi(arg), VIRTUAL);
	char_to_room(victim, ch->in_room);
}
 

void do_mpoload(struct char_data *ch, char *argument)
{
	char arg1[MAX_INPUT_LENGTH];
	struct index_data *pObjIndex;
	struct obj_data       *obj;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	argument = one_argument(argument, arg1);
  
	if (arg1[0] == '\0' || !is_number(arg1)) {
		log("Mpoload - Bad syntax: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
  
	if ((pObjIndex = get_obj_index(atoi(arg1))) == NULL) {
		log("Mpoload - Bad vnum arg: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	obj = read_object(atoi(arg1), VIRTUAL);
	if (CAN_WEAR(obj, ITEM_WEAR_TAKE)) 
		obj_to_char(obj, ch);
	else
		obj_to_room(obj, IN_ROOM(ch));
}
 

/* Let the mobile purge all objects and other npcs in the room, or
 * purge a specified object or mob in the room.  It can purge itself,
 * but this had best be the last command in the MOBprogram otherwise
 * ugly stuff will happen */
void do_mppurge(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *victim, *vnext;
	struct obj_data  *obj,  *obj_next;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	one_argument(argument, arg);
 
	if (arg[0] == '\0') {
		/* 'purge' */
 
		for (victim = world[IN_ROOM(ch)].people; victim != NULL; victim = vnext) {
			vnext = victim->next_in_room;
			if (IS_NPC(victim) && victim != ch)
				extract_char(victim);
		}
 
		for (obj = world[IN_ROOM(ch)].contents; obj != NULL; obj = obj_next) {
			obj_next = obj->next_content;
			extract_obj(obj);
		}
 
		return;
	}
 
	if ((victim = get_char_room_vis(ch, arg, NULL))) {
		if ((obj = get_obj_vis(ch, arg, NULL)))
			extract_obj(obj);
		else
			log("Mppurge - Bad argument: vnum %d.",mob_index[ch->nr].vnum);

		return;
	}
 
	if (!IS_NPC(victim)) {
		log("Mppurge - Purging a PC: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	extract_char(victim);
}
 
 
/* lets the mobile goto any location it wishes that is not private */
void do_mpgoto(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
	room_vnum location;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	one_argument(argument, arg);
	if (arg[0] == '\0') {
		log("Mpgoto - No argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if ((location = find_target_room(ch, arg)) < 0) {
		log("Mpgoto - No such location: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (FIGHTING(ch))
		stop_fighting(ch);
 
	char_from_room(ch);
	char_to_room(ch, location);
}
 

/* lets the mobile do a command at another location. Very useful */
void do_mpat(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
	room_vnum location;
	room_vnum original;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
  
	argument = one_argument(argument, arg);
 
	if (arg[0] == '\0' || argument[0] == '\0') {
		log("Mpat - Bad argument: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if ((location = find_target_room(ch, arg)) < 0) {
		log("Mpat - No such location: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	original = IN_ROOM(ch);
	char_from_room(ch);
	char_to_room(ch, location);
	command_interpreter(ch, argument);
 
	/*
	 * See if 'ch' still exists before continuing!
	 * Handles 'at XXXX quit' case.
	 */
	if (IN_ROOM(ch) == location) {
		char_from_room(ch);
		char_to_room(ch, original);
	}
}
  

/* Let the mobile transfer people.  The all argument transfers
 * everyone in the current room to the specified location */
void do_mptransfer(struct char_data *ch, char *argument)
{
	char arg1[MAX_INPUT_LENGTH];
	char arg2[MAX_INPUT_LENGTH];
	room_vnum location;
	struct descriptor_data *d;
	struct char_data *victim;
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
	argument = one_argument(argument, arg1);
	argument = one_argument(argument, arg2);
 
	if (arg1[0] == '\0') {
		log("Mptransfer - Bad syntax: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (!str_cmp(arg1, "all")) {
		for (d = descriptor_list; d != NULL; d = d->next) {
			if (d->connected == CON_PLAYING &&
				 d->character != ch &&
				 IN_ROOM(d->character) != NOWHERE &&
				 CAN_SEE(ch, d->character)) {
				char buf[MAX_STRING_LENGTH];
				sprintf(buf, "%s %s", d->character->player.name, arg2);
				do_trans(ch, buf, 0, 0);
			}
		}
		return;
	}
 
	/*
	 * Thanks to Grodyn for the optional location parameter.
	 */
	if (arg2[0] == '\0')
		location = IN_ROOM(ch);
	else {
		if ((location = find_target_room(ch, arg2)) < 0) {
			log("Mptransfer - No such location: vnum %d.",
				  mob_index[ch->nr].vnum);
			return;
		}
 
		if (ROOM_FLAGGED(location, ROOM_PRIVATE)) {
			log("Mptransfer - Private room: vnum %d.",
				  mob_index[ch->nr].vnum);
			return;
		}
	}
 
	if ((victim = get_char_vis(ch, arg1, NULL, FIND_CHAR_WORLD)) == NULL) {
		log("Mptransfer - No such person: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (IN_ROOM(victim) == NOWHERE) {
		log("Mptransfer - Victim in Limbo: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (FIGHTING(victim)) 
		stop_fighting(victim);
 
	char_from_room(victim);
	char_to_room(victim, location);
}
 

/* Let the mobile force someone to do something.  must be mortal level
 *	and the all argument only affects those in the room with the
 *	mobile. */
void do_mpforce(struct char_data *ch, char *argument)
{
	char arg[MAX_INPUT_LENGTH];
 
	if (!IS_NPC(ch)) {
		send_to_char(ch, "Huh?\r\n");
		return;
	}
 
	argument = one_argument(argument, arg);
 
	if (arg[0] == '\0' || argument[0] == '\0') {
		log("Mpforce - Bad syntax: vnum %d.", mob_index[ch->nr].vnum);
		return;
	}
 
	if (!str_cmp(arg, "all")) {
		struct descriptor_data *i;
		struct char_data *vch;
 
		for (i = descriptor_list; i ; i = i->next) {
			if (i->character != ch && !i->connected &&
				IN_ROOM(i->character) == IN_ROOM(ch)) {
				vch = i->character;

				if (GET_LEVEL(vch) >= LVL_IMMORT)
					continue;
				
				if (GET_LEVEL(vch) < GET_LEVEL(ch) && CAN_SEE(ch, vch))
					command_interpreter(vch, argument);
			}
		}
	} else {
		struct char_data *victim;
 
		if ((victim = get_char_room_vis(ch, arg, NULL)) == NULL) {
			log("Mpforce - No such victim: vnum %d.",
				  mob_index[ch->nr].vnum);
			return;
		}
 
		if (victim == ch) {
			log("Mpforce - Forcing oneself: vnum %d.",
				  mob_index[ch->nr].vnum);
			return;
		}

		if (GET_LEVEL(victim) >= LVL_IMMORT) {
			log("Mpforce - Forcing an immort: vnum %d.",
				  mob_index[ch->nr].vnum);
			return;
		}
		
		command_interpreter(victim, argument);
	}
}
