/************************************************************************
 *  --Statedit  Part of UrathMud                                v1.0    *
 *  Copyright 1999 Karl N. Matthias.  All rights Reserved.              *
 *  You may freely distribute, modify, or sell this code                *
 *  as long as this copyright remains intact.                           *
 *                                                                      *
 *  Based on code by Jeremy Elson, Harvey Gilpin, and George Greer.     *
 ************************************************************************/

/* --relistan 2/22/99 - 2/24/99 */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "olc.h"

int parse_stat_menu(struct descriptor_data *d, char *arg);
int parse_stats(struct descriptor_data *d, char *arg);
int stats_disp_menu(struct descriptor_data *d);
void init_stats(struct descriptor_data *d);
int stats_assign_stat(int abil, char *arg, struct descriptor_data *d);

/* --relistan 2/22/99 for player configurable stats */
int parse_stats(struct descriptor_data *d, char *arg)
{
	struct char_data *ch;

	ch = d->character;

	switch (OLC_MODE(d)) {
		case STAT_QUIT:
			return (TRUE);

		case STAT_PARSE_MENU:
			if (parse_stat_menu(d, arg)) 
				return (TRUE);
			break;

		case STAT_GET_STR:
			ch->real_abils.str = stats_assign_stat(ch->real_abils.str, arg, d);
			stats_disp_menu(d);
			break;

		case STAT_GET_INT:
			ch->real_abils.intel = stats_assign_stat(ch->real_abils.intel, arg, d);
			stats_disp_menu(d);
			break;

		case STAT_GET_WIS:
			ch->real_abils.wis = stats_assign_stat(ch->real_abils.wis, arg, d);
			stats_disp_menu(d);
			break;

		case STAT_GET_DEX:
			ch->real_abils.dex = stats_assign_stat(ch->real_abils.dex, arg, d);
			stats_disp_menu(d);
			break;

		case STAT_GET_CON:
			ch->real_abils.con = stats_assign_stat(ch->real_abils.con, arg, d);
			stats_disp_menu(d);
			break;

		case STAT_GET_CHA:
			ch->real_abils.cha = stats_assign_stat(ch->real_abils.cha, arg, d);
			stats_disp_menu(d);
			break;

		default:
			OLC_MODE(d) = stats_disp_menu(d); 
			break;
	}

	return (FALSE);
}


/* Roll up the initial stats and assign them to OLC_VAL(d).  This
 * isn''t nearly as efficient as it could be, but this code is horked
 * from the original roll_real_abils() code */
void init_stats(struct descriptor_data *d)
{
	int i, j, k, temp;
	ubyte table[6];
	ubyte rolls[4];

	for (i = 0; i < 6; i++)
		table[i] = 0;

	for (i = 0; i < 6; i++) {

		for (j = 0; j < 4; j++)
			rolls[j] = rand_number(1, 6);

		temp = rolls[0] + rolls[1] + rolls[2] + rolls[3] -
			MIN(rolls[0], MIN(rolls[1], MIN(rolls[2], rolls[3])));

		for (k = 0; k < 6; k++)
			if (table[k] < temp) {
				temp ^= table[k];
				table[k] ^= temp;
				temp ^= table[k];
			}
	}

	OLC_VAL(d) = table[0] + table[1] + table[2] + table[3] + table[4] + table[5];

	d->character->real_abils.str_add = 0;

	/* Minimum stats 8 */
	d->character->real_abils.str = 8;
	d->character->real_abils.intel = 8;
	d->character->real_abils.cha = 8;
	d->character->real_abils.dex = 8;
	d->character->real_abils.con = 8;
	d->character->real_abils.wis = 8;

	OLC_VAL(d) -= 48;

	stats_disp_menu(d);
}


int stats_disp_menu(struct descriptor_data *d)
{
	send_to_char(d->character, 
		"\r\n"
		"-<[=========[ MUD ]=========]>-\r\n"
		" <| Total Points Left: %3d  |>     You should select the letter of\r\n"
		" <|                         |>    the score you wish to adjust.\r\n"
		" <| = Select a stat:        |>    When prompted, enter the new\r\n"
		" <| S) Strength     : %2d    |>    score, NOT the amount to add.\r\n"
		" <| I) Intelligence : %2d    |>    NOTE: If you quit before you\r\n"
		" <| W) Wisdom       : %2d    |>    assign all the points, you\r\n"
		" <| D) Dexterity    : %2d    |>    will lose them forever.\r\n"
		" <| N) Constitution : %2d    |>\r\n"
		" <| C) Charisma     : %2d    |>\r\n"
		" <| Q) Quit                 |>\r\n"
		"-<[=========================]>-\r\n"
		"\r\n", OLC_VAL(d),
		d->character->real_abils.str,
		d->character->real_abils.intel,
		d->character->real_abils.wis,
		d->character->real_abils.dex,
		d->character->real_abils.con,
		d->character->real_abils.cha);

	send_to_char(d->character, "Enter Letter to Change: ");
	OLC_MODE(d) = STAT_PARSE_MENU;

	return (TRUE);
}


int parse_stat_menu(struct descriptor_data *d, char *arg)
{
	struct char_data *ch = d->character;
	
	/* Main parse loop */
	*arg = LOWER(*arg);
	switch (*arg) {
		case 's':
			OLC_MODE(d) = STAT_GET_STR; 
			send_to_char(ch, "Enter New value: ");
			break;
		case 'i':
			OLC_MODE(d) = STAT_GET_INT; 
			send_to_char(ch, "Enter New value: ");
			break;
		case 'w':
			OLC_MODE(d) = STAT_GET_WIS;  
			send_to_char(ch, "Enter New value: ");
			break;
		case 'd':
			OLC_MODE(d) = STAT_GET_DEX;  
			send_to_char(ch, "Enter New value: ");
			break;
		case 'n':
			OLC_MODE(d) = STAT_GET_CON;  
			send_to_char(ch, "Enter New value: ");
			break; 
		case 'c':
			OLC_MODE(d) = STAT_GET_CHA;  
			send_to_char(ch, "Enter New value: ");
			break;
		case 'q':
			OLC_MODE(d) = STAT_QUIT; 
			return (TRUE);
  
		default:
			stats_disp_menu(d);
	}
	return (FALSE);
}


int stats_assign_stat(int abil, char *arg, struct descriptor_data *d)
{
	int temp;

	if (abil > 0) {
      OLC_VAL(d) = OLC_VAL(d) + abil; 
      abil = 0;
	}

	if (atoi(arg) > OLC_VAL(d)) 
		temp = OLC_VAL(d);
	else
		temp = atoi(arg);

	if (temp > 18) {
		if (OLC_VAL(d) < 18)
			temp = OLC_VAL(d);
		else 
			temp = 18;
	}

	if (temp < 8)
		temp = 8;

	/* This should throw an error! */
	if (OLC_VAL(d) <= 0) {
		temp = 0;
		OLC_VAL(d) = 0;

		mudlog(NRM, LVL_IMMORT, TRUE, "Stat total below 0: possible code error");
	}
	abil = temp;
	OLC_VAL(d) -= temp;

	return (abil);
}
